@@ -225,64 +225,70 @@ typedef struct faslFileObj {
 } *faslFile;
 
 /* locally defined functions */
-static INT uf_read PROTO((unbufFaslFile uf, octet *s, iptr n));
-static octet uf_bytein PROTO((unbufFaslFile uf));
-static uptr uf_uptrin PROTO((unbufFaslFile uf, INT *bytes_consumed));
-static ptr fasl_entry PROTO((ptr tc, IFASLCODE situation, unbufFaslFile uf, ptr externals));
-static ptr bv_fasl_entry PROTO((ptr tc, ptr bv, IFASLCODE ty, uptr offset, uptr len, unbufFaslFile uf, ptr externals));
-static void fillFaslFile PROTO((faslFile f));
-static void bytesin PROTO((octet *s, iptr n, faslFile f));
-static void toolarge PROTO((ptr path));
-static iptr iptrin PROTO((faslFile f));
-static uptr uptrin PROTO((faslFile f));
-static float singlein PROTO((faslFile f));
-static double doublein PROTO((faslFile f));
-static iptr stringin PROTO((ptr *pstrbuf, iptr start, faslFile f));
-static void faslin PROTO((ptr tc, ptr *x, ptr t, ptr *pstrbuf, faslFile f));
-static void fasl_record PROTO((ptr tc, ptr *x, ptr t, ptr *pstrbuf, faslFile f, uptr size));
-static IBOOL rtd_equiv PROTO((ptr x, ptr y));
-static IBOOL equalp PROTO((ptr x, ptr y));
+static INT uf_read(unbufFaslFile uf, octet *s, iptr n);
+static octet uf_bytein(unbufFaslFile uf);
+static uptr uf_uptrin(unbufFaslFile uf, INT *bytes_consumed);
+static ptr fasl_entry(ptr tc, IFASLCODE situation, unbufFaslFile uf, ptr externals);
+static ptr bv_fasl_entry(ptr tc, ptr bv, IFASLCODE ty, uptr offset, uptr len, unbufFaslFile uf, ptr externals);
+static void fillFaslFile(faslFile f);
+static void bytesin(octet *s, iptr n, faslFile f);
+static void toolarge(ptr path);
+static iptr iptrin(faslFile f);
+static uptr uptrin(faslFile f);
+static float singlein(faslFile f);
+static double doublein(faslFile f);
+static iptr stringin(ptr *pstrbuf, iptr start, faslFile f);
+static void faslin(ptr tc, ptr *x, ptr t, ptr *pstrbuf, faslFile f);
+static void fasl_record(ptr tc, ptr *x, ptr t, ptr *pstrbuf, faslFile f, uptr size);
+static IBOOL rtd_equiv(ptr x, ptr y);
+static IBOOL equalp(ptr x, ptr y);
 #ifdef PORTABLE_BYTECODE
-static void pb_set_abs PROTO((void *address, uptr item));
-static uptr pb_get_abs PROTO((void *address));
+static void pb_set_abs(void *address, uptr item);
+static uptr pb_get_abs(void *address);
 #endif /* AARCH64 */
 #ifdef ARMV6
-static void arm32_set_abs PROTO((void *address, uptr item));
-static uptr arm32_get_abs PROTO((void *address));
-static void arm32_set_jump PROTO((void *address, uptr item, IBOOL callp));
-static uptr arm32_get_jump PROTO((void *address));
+static void arm32_set_abs(void *address, uptr item);
+static uptr arm32_get_abs(void *address);
+static void arm32_set_jump(void *address, uptr item, IBOOL callp);
+static uptr arm32_get_jump(void *address);
 #endif /* ARMV6 */
 #ifdef AARCH64
-static void arm64_set_abs PROTO((void *address, uptr item));
-static uptr arm64_get_abs PROTO((void *address));
+static void arm64_set_abs(void *address, uptr item);
+static uptr arm64_get_abs(void *address);
 #endif /* AARCH64 */
 #ifdef PPC32
-static void ppc32_set_abs PROTO((void *address, uptr item));
-static uptr ppc32_get_abs PROTO((void *address));
-static void ppc32_set_jump PROTO((void *address, uptr item, IBOOL callp));
-static uptr ppc32_get_jump PROTO((void *address));
+static void ppc32_set_abs(void *address, uptr item);
+static uptr ppc32_get_abs(void *address);
+static void ppc32_set_jump(void *address, uptr item, IBOOL callp);
+static uptr ppc32_get_jump(void *address);
 #endif /* PPC32 */
 #ifdef X86_64
-static void x86_64_set_jump PROTO((void *address, uptr item, IBOOL callp));
-static uptr x86_64_get_jump PROTO((void *address));
-static void x86_64_set_popcount PROTO((void *address, uptr item));
+static void x86_64_set_jump(void *address, uptr item, IBOOL callp);
+static uptr x86_64_get_jump(void *address);
+static void x86_64_set_popcount(void *address, uptr item);
 #endif /* X86_64 */
 #ifdef SPARC64
-static INT extract_reg_from_sethi PROTO((void *address));
-static void emit_sethi_lo PROTO((U32 item, INT destreg, void *address));
-static uptr sparc64_get_literal PROTO((void *address));
-static void sparc64_set_call PROTO((void *address, U32 *call_addr, uptr item));
-static U32 adjust_delay_inst PROTO((U32 delay_inst, U32 *old_call_addr, U32 *new_call_addr));
-static INT sparc64_set_lit_only PROTO((void *address, uptr item, I32 destreg));
-static void sparc64_set_literal PROTO((void *address, uptr item));
+static INT extract_reg_from_sethi(void *address);
+static void emit_sethi_lo(U32 item, INT destreg, void *address);
+static uptr sparc64_get_literal(void *address);
+static void sparc64_set_call(void *address, U32 *call_addr, uptr item);
+static U32 adjust_delay_inst(U32 delay_inst, U32 *old_call_addr, U32 *new_call_addr);
+static INT sparc64_set_lit_only(void *address, uptr item, I32 destreg);
+static void sparc64_set_literal(void *address, uptr item);
 #endif /* SPARC64 */
-#ifdef PORTABLE_BYTECODE_BIGENDIAN
+#ifdef RISCV64
+static void riscv64_set_abs(void *address, uptr item);
+static uptr riscv64_get_abs(void *address);
+static void riscv64_set_jump(void *address, uptr item);
+static uptr riscv64_get_jump(void *address);
+#endif /* RISCV64 */
+#ifdef PORTABLE_BYTECODE_SWAPENDIAN
 static void swap_code_endian(octet *code, uptr len);
 #endif
 
 static double s_nan;
 
-void S_fasl_init() {
+void S_fasl_init(void) {
     if (S_boot_time) {
         S_protect(&S_G.base_rtd);
         S_G.base_rtd = Sfalse;
@@ -830,10 +836,14 @@ static void faslin(ptr tc, ptr *x, ptr t, ptr *pstrbuf, faslFile f) {
             }
             return;
         }
-        case fasl_type_stencil_vector: {
+        case fasl_type_stencil_vector:
+        case fasl_type_system_stencil_vector: {
             uptr mask; iptr n; ptr *p;
             mask = uptrin(f);
-            *x = S_stencil_vector(mask);
+            if (ty == fasl_type_stencil_vector)
+              *x = S_stencil_vector(mask);
+            else
+              *x = S_system_stencil_vector(mask);
             p = &INITSTENVECTIT(*x, 0);
             n = Spopcount(mask);
             while (n--) faslin(tc, p++, t, pstrbuf, f);
@@ -1071,7 +1081,7 @@ static void faslin(ptr tc, ptr *x, ptr t, ptr *pstrbuf, faslFile f) {
               S_G.profile_counters = Scons(S_weak_cons(co, pinfos), S_G.profile_counters);
             }
             code_bytesin((octet *)&CODEIT(co, 0), n, f);
-#ifdef PORTABLE_BYTECODE_BIGENDIAN
+#ifdef PORTABLE_BYTECODE_SWAPENDIAN
             swap_code_endian((octet *)&CODEIT(co, 0), n);
 #endif
             m = uptrin(f);
@@ -1323,7 +1333,7 @@ int S_fasl_intern_rtd(ptr *x)
 }
 
 /* limited version for checking rtd fields */
-static IBOOL equalp(x, y) ptr x, y; {
+static IBOOL equalp(ptr x, ptr y) {
   if (x == y) return 1;
   if (Spairp(x)) return Spairp(y) && equalp(Scar(x), Scar(y)) && equalp(Scdr(x), Scdr(y));
   if (Svectorp(x)) {
@@ -1336,7 +1346,7 @@ static IBOOL equalp(x, y) ptr x, y; {
   return Sbignump(x) && Sbignump(y) && S_big_eq(x, y);
 }
 
-static IBOOL rtd_equiv(x, y) ptr x, y; {
+static IBOOL rtd_equiv(ptr x, ptr y) {
   return ((RECORDINSTTYPE(x) == RECORDINSTTYPE(y))
           /* recognize `base-rtd` shape: */
           || ((RECORDINSTTYPE(x) == x)
@@ -1378,14 +1388,14 @@ INT pax_encode21(INT n)
 #endif /* HPUX */
 
 /* used here, in S_gc(), and in compile.ss */
-void S_set_code_obj(who, typ, p, n, x, o) char *who; IFASLCODE typ; iptr n, o; ptr p, x; {
+void S_set_code_obj(char *who, IFASLCODE typ, ptr p, iptr n, ptr x, iptr o) {
     void *address; uptr item;
 
     address = TO_VOIDP((uptr)p + n);
     item = (uptr)x + o;
     switch (typ) {
         case reloc_abs:
-            *(uptr *)address = item;
+            STORE_UNALIGNED_UPTR(address, item);
             break;
 #ifdef PORTABLE_BYTECODE
         case reloc_pb_abs:
@@ -1466,19 +1476,28 @@ void S_set_code_obj(who, typ, p, n, x, o) char *who; IFASLCODE typ; iptr n, o; p
             *(U32 *)address = *(U32 *)address & ~0x3fffffff | item >> 2 & 0x3fffffff;
             break;
 #endif /* SPARC */
+#ifdef RISCV64
+        case reloc_riscv64_abs:
+        case reloc_riscv64_call:
+          riscv64_set_abs(address, item);
+          break;
+        case reloc_riscv64_jump:
+          riscv64_set_jump(address, item);
+          break;
+#endif /* RISCV64 */
         default:
             S_error1(who, "invalid relocation type ~s", FIX(typ));
     }
 }
 
 /* used in S_gc() */
-ptr S_get_code_obj(typ, p, n, o) IFASLCODE typ; iptr n, o; ptr p; {
+ptr S_get_code_obj(IFASLCODE typ, ptr p, iptr n, iptr o) {
     void *address; uptr item;
 
     address = TO_VOIDP((uptr)p + n);
     switch (typ) {
         case reloc_abs:
-            item = *(uptr *)address;
+            item = LOAD_UNALIGNED_UPTR(address);
             break;
 #ifdef PORTABLE_BYTECODE
         case reloc_pb_abs:
@@ -1546,6 +1565,15 @@ ptr S_get_code_obj(typ, p, n, o) IFASLCODE typ; iptr n, o; ptr p; {
             item += (uptr)address;
             break;
 #endif /* SPARC */
+#ifdef RISCV64 //@ todo
+        case reloc_riscv64_abs:
+        case reloc_riscv64_call:
+          item = riscv64_get_abs(address);
+          break;
+        case reloc_riscv64_jump:
+          item = riscv64_get_jump(address);
+          break;
+#endif /* RISCV64 */
         default:
             S_error1("", "invalid relocation type ~s", FIX(typ));
             return (ptr)0 /* not reached */;
@@ -1555,39 +1583,12 @@ ptr S_get_code_obj(typ, p, n, o) IFASLCODE typ; iptr n, o; ptr p; {
 
 #ifdef PORTABLE_BYTECODE
 
-/* Address pieces in a movz,movk,movk,movk sequence are upper 16 bits */
-#define ADDRESS_BITS_SHIFT 16
-#define ADDRESS_BITS_MASK  ((U32)0xFFFF0000)
-#define DEST_REG_MASK      0xF00
-
 static void pb_set_abs(void *address, uptr item) {
-  /* First word can have an arbitrary value due to vfasl offset
-     storage, so get the target register from the end: */
-#if ptr_bytes == 8  
-  int dest_reg = ((U32 *)address)[3] & DEST_REG_MASK;
-#else
-  int dest_reg = ((U32 *)address)[1] & DEST_REG_MASK;
-#endif
-
-  /* pb_link is the same as pb_mov16_pb_zero_bits_pb_shift0, but with
-     a promise of the subsequent instructions to load a full word */
-
-  ((U32 *)address)[0] = (pb_link | dest_reg | ((item & 0xFFFF) << ADDRESS_BITS_SHIFT));
-  ((U32 *)address)[1] = (pb_mov16_pb_keep_bits_pb_shift1 | dest_reg | (((item >> 16) & 0xFFFF) << ADDRESS_BITS_SHIFT));
-#if ptr_bytes == 8  
-  ((U32 *)address)[2] = (pb_mov16_pb_keep_bits_pb_shift2 | dest_reg | (((item >> 32) & 0xFFFF) << ADDRESS_BITS_SHIFT));
-  ((U32 *)address)[3] = (pb_mov16_pb_keep_bits_pb_shift3 | dest_reg | (((item >> 48) & 0xFFFF) << ADDRESS_BITS_SHIFT));
-#endif
+  STORE_UNALIGNED_UPTR(address, item);
 }
 
 static uptr pb_get_abs(void *address) {
-  return ((uptr)((((U32 *)address)[0] & ADDRESS_BITS_MASK) >> ADDRESS_BITS_SHIFT)
-          | ((uptr)((((U32 *)address)[1] & ADDRESS_BITS_MASK) >> ADDRESS_BITS_SHIFT) << 16)
-#if ptr_bytes == 8          
-          | ((uptr)((((U32 *)address)[2] & ADDRESS_BITS_MASK) >> ADDRESS_BITS_SHIFT) << 32)
-          | ((uptr)((((U32 *)address)[3] & ADDRESS_BITS_MASK) >> ADDRESS_BITS_SHIFT) << 48)
-#endif
-          );
+  return LOAD_UNALIGNED_UPTR(address);
 }
 
 #endif /* PORTABLE_BYTECODE */
@@ -1663,7 +1664,7 @@ static uptr arm32_get_jump(void *address) {
 #define SHIFT48_OPCODE 0x00600000
 
 static void arm64_set_abs(void *address, uptr item) {
-  /* First word can have an arbitrary value due to vfasl offset
+  /* First instruction can have an arbitrary value due to vfasl offset
      storage, so get the target register from the end: */
   int dest_reg = ((U32 *)address)[3] & DEST_REG_MASK;
   
@@ -1840,7 +1841,7 @@ void x86_64_set_popcount_present(ptr code) {
 #define CALL(disp) (OP_CALL | (disp) >> 2 & 0x3fffffff)
 
 
-static INT extract_reg_from_sethi(address) void *address; {
+static INT extract_reg_from_sethi(void* address) {
   return *(U32 *)address >> 25;
 }
 
@@ -1854,7 +1855,7 @@ static void emit_sethi_lo(U32 item, INT destreg, void *address) {
   *((U32 *)address + 1) = ORI(destreg,low,destreg);
 }
 
-static uptr sparc64_get_literal(address) void *address; {
+static uptr sparc64_get_literal(void *address) {
   uptr item;
 
  /* we may have "call disp" followed by delay instruction */
@@ -1900,7 +1901,7 @@ static U32 adjust_delay_inst(delay_inst, old_call_addr, new_call_addr)
   return 0; /* fortunately, not a valid instruction here */
 }
 
-static void sparc64_set_call(address, call_addr, item) void *address; U32 *call_addr; uptr item; {
+static void sparc64_set_call(void *address, U32 *call_addr, uptr item) {
   U32 delay_inst = *(call_addr + 1), new_delay_inst; iptr disp;
 
  /* later: make item local if it refers to Scheme code, i.e., is in the
@@ -1920,7 +1921,7 @@ static void sparc64_set_call(address, call_addr, item) void *address; U32 *call_
   }
 }
 
-static INT sparc64_set_lit_only(address, item, destreg) void *address; uptr item; I32 destreg; {
+static INT sparc64_set_lit_only(void *address, uptr item, I32 destreg) {
 
   if ((iptr)item >= -0xffffffff && item <= 0xffffffff) {
     uptr x, high, low;
@@ -1953,7 +1954,7 @@ static INT sparc64_set_lit_only(address, item, destreg) void *address; uptr item
   }
 }
 
-static void sparc64_set_literal(address, item) void *address; uptr item; {
+static void sparc64_set_literal(void* address, uptr item) {
   I32 destreg;
 
  /* case 1: we have call followed by delay inst */
@@ -1984,7 +1985,64 @@ static void sparc64_set_literal(address, item) void *address; uptr item; {
 }
 #endif /* SPARC64 */
 
-#ifdef PORTABLE_BYTECODE_BIGENDIAN
+#ifdef RISCV64
+static uptr riscv64_get_abs(void* address)
+{
+  return *((I64 *)((I32 *)address + 3));
+}
+
+static uptr riscv64_get_jump(void* address)
+{
+  return *((I64 *)((I32 *)address + 3));
+}
+
+#define AUIPC_INSTR(dest, offset)   (0x17 | ((dest) << 7) | ((offset) << 12))
+#define LD_INSTR(dest, src, offset) (0x03 | ((dest) << 7) | (3 << 12) | ((src) << 15) | ((offset) << 20))
+#define EXTRACT_LD_INSTR_DEST(instr) ((instr >> 7) & 0x1F)
+#define REAL_ZERO_REG 0
+#define JUMP_REG 30
+
+static void riscv64_set_abs(void* address, uptr item)
+{
+  /*
+     Code sequence:
+
+     [0] auipc dest, 0
+     [1] ld dest, dest, 16
+     [2] jal %real-zero, 16
+     [3-4] 8-bytes of addr
+     [5] jalr - in case of call
+
+    Although vfasl may have overwritten the first instruction,
+    so we need to extract `dest` from the second.
+  */
+  int dest = EXTRACT_LD_INSTR_DEST(((I32 *)address)[1]);
+  ((I32 *)address)[0] = AUIPC_INSTR(dest, 0);
+
+  (*((I64 *)((I32 *)address + 3))) = item;
+}
+
+static void riscv64_set_jump(void* address, uptr item)
+{
+  /*
+     Code sequence:
+
+    [0] auipc %jump, 0
+    [1] ld %jump, %jump, 12
+    [2] jal %real_zero, 12
+    [3] 8-bytes of addr
+    [5] jalr
+
+    Although vfasl may have overwritten the first instruction,
+    it's always the same, so we can reconstruct it.
+  */
+  ((I32 *)address)[0] = AUIPC_INSTR(JUMP_REG, 0);
+
+  (*((I64 *)((I32 *)address + 3))) = item;
+}
+#endif /* RISCV64 */
+
+#ifdef PORTABLE_BYTECODE_SWAPENDIAN
 typedef struct {
   octet *code;
   uptr size;
@@ -2028,6 +2086,13 @@ static void swap_code_endian(octet *code, uptr len)
       octet b = code[1];
       octet c = code[2];
       octet d = code[3];
+#if fasl_endianness_is_little
+      octet le_a = a, le_b = b, le_c = c, le_d = d;
+# define le_tag_offset -ptr_bytes
+#else
+      octet le_a = d, le_b = c, le_c = b, le_d = a;
+# define le_tag_offset -1
+#endif
       code[0] = d;
       code[1] = c;
       code[2] = b;
@@ -2036,9 +2101,9 @@ static void swap_code_endian(octet *code, uptr len)
       code += 4;
       len -= 4;
 
-      if (a == pb_adr) {
+      if (le_a == pb_adr) {
         /* delta can be negative for a mvlet-error reinstall of the return address */
-        iptr delta = (((iptr)d << (ptr_bits - 8)) >> (ptr_bits - 20)) + ((iptr)c << 4) + (b >> 4);
+        iptr delta = 4*((((iptr)le_d << (ptr_bits - 8)) >> (ptr_bits - 20)) + ((iptr)le_c << 4) + (le_b >> 4));
         if (delta > 0) {
           /* after a few more instructions, we'll hit
              a header where 64-bit values needs to be
@@ -2049,15 +2114,13 @@ static void swap_code_endian(octet *code, uptr len)
 
 	  if ((uptr)delta > len)
 	    S_error_abort("swap endian: delta goes past end");
-	  if (delta & 0x3)
-	    S_error_abort("swap endian: delta is not a multiple of 4");
 
-          if (after_rpheader[-8] & 0x1)
+          if (after_rpheader[le_tag_offset] & 0x1)
             header_size = size_rp_compact_header;
           else
             header_size = size_rp_header;
           rpheader = after_rpheader - header_size;
-
+          
 	  if (rpheader_stack_pos == rpheader_stack_size) {
 	    int new_size = (2 * rpheader_stack_size) + 16;
 	    rpheader_t *new_stack;
