@@ -25,33 +25,33 @@
 #include "system.h"
 
 /* locally defined functions */
-static ptr copy_normalize PROTO((ptr tc, const bigit *p, iptr len, IBOOL sign));
-static IBOOL abs_big_lt PROTO((ptr x, ptr y, iptr xl, iptr yl));
-static IBOOL abs_big_eq PROTO((ptr x, ptr y, iptr xl, iptr yl));
-static ptr big_negate PROTO((ptr tc, ptr x));
-static ptr big_add_pos PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign));
-static ptr big_add_neg PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign));
-static ptr big_add PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys));
-static ptr big_mul PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign));
-static void big_short_trunc PROTO((ptr tc, ptr x, bigit s, iptr xl, IBOOL qs, IBOOL rs, ptr *q, ptr *r));
-static void big_trunc PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL qs, IBOOL rs, ptr *q, ptr *r));
-static INT normalize PROTO((bigit *xp, bigit *yp, iptr xl, iptr yl));
-static bigit quotient_digit PROTO((ptr tc, bigit *xp, bigit *yp, iptr yl));
-static bigit qhat PROTO((bigit *xp, bigit *yp));
-static ptr big_short_gcd PROTO((ptr tc, ptr x, bigit y, iptr xl));
-static ptr big_gcd PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl));
-static ptr s_big_ash PROTO((ptr tc, bigit *xp, iptr xl, IBOOL sign, iptr cnt));
-static double big_short_floatify PROTO((ptr tc, ptr x, bigit s, iptr xl, IBOOL sign));
-static double big_floatify PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign));
-static double floatify_normalize PROTO((bigit *p, iptr e, IBOOL sign, IBOOL sticky));
-static double floatify_ratnum PROTO((ptr tc, ptr x));
-static ptr big_logbitp PROTO((iptr n, ptr x, iptr xl, IBOOL xs));
-static ptr big_logbit0 PROTO((ptr tc, ptr origx, iptr n, ptr x, iptr xl, IBOOL xs));
-static ptr big_logbit1 PROTO((ptr tc, ptr origx, iptr n, ptr x, iptr xl, IBOOL xs));
-static ptr big_logtest PROTO((ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys));
-static ptr big_logand PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys));
-static ptr big_logor PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys));
-static ptr big_logxor PROTO((ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys));
+static ptr copy_normalize(ptr tc, const bigit *p, iptr len, IBOOL sign);
+static IBOOL abs_big_lt(ptr x, ptr y, iptr xl, iptr yl);
+static IBOOL abs_big_eq(ptr x, ptr y, iptr xl, iptr yl);
+static ptr big_negate(ptr tc, ptr x);
+static ptr big_add_pos(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign);
+static ptr big_add_neg(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign);
+static ptr big_add(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys);
+static ptr big_mul(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign);
+static void big_short_trunc(ptr tc, ptr x, bigit s, iptr xl, IBOOL qs, IBOOL rs, ptr *q, ptr *r);
+static void big_trunc(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL qs, IBOOL rs, ptr *q, ptr *r);
+static INT normalize(bigit *xp, bigit *yp, iptr xl, iptr yl);
+static bigit quotient_digit(ptr tc, bigit *xp, bigit *yp, iptr yl);
+static bigit qhat(bigit *xp, bigit *yp);
+static ptr big_short_gcd(ptr tc, ptr x, bigit y, iptr xl);
+static ptr big_gcd(ptr tc, ptr x, ptr y, iptr xl, iptr yl);
+static ptr s_big_ash(ptr tc, bigit *xp, iptr xl, IBOOL sign, iptr cnt);
+static double big_short_floatify(ptr tc, ptr x, bigit s, iptr xl, IBOOL sign);
+static double big_floatify(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign);
+static double floatify_normalize(bigit *p, iptr e, IBOOL sign, IBOOL sticky);
+static double floatify_ratnum(ptr tc, ptr x);
+static ptr big_logbitp(iptr n, ptr x, iptr xl, IBOOL xs);
+static ptr big_logbit0(ptr tc, ptr origx, iptr n, ptr x, iptr xl, IBOOL xs);
+static ptr big_logbit1(ptr tc, ptr origx, iptr n, ptr x, iptr xl, IBOOL xs);
+static ptr big_logtest(ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys);
+static ptr big_logand(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys);
+static ptr big_logor(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys);
+static ptr big_logxor(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys);
 
 /* use w/o trailing semicolon */
 #define PREPARE_BIGNUM(tc,x,l)\
@@ -164,7 +164,7 @@ ptr S_normalize_bignum(ptr x) {
   return x;
 }
 
-static ptr copy_normalize(tc, p, len, sign) ptr tc; const bigit *p; iptr len; IBOOL sign; {
+static ptr copy_normalize(ptr tc, const bigit *p, iptr len, IBOOL sign) {
   bigit *p1; uptr n; ptr b;
 
   for (;;) {
@@ -203,7 +203,7 @@ static ptr copy_normalize(tc, p, len, sign) ptr tc; const bigit *p; iptr len; IB
 }
 
 /* -2^(b-1) <= x <= 2^b-1, where b = number of bits in a uptr */
-iptr S_integer_value(who, x) const char *who; ptr x; {
+iptr S_integer_value(const char *who, ptr x) {
   if (Sfixnump(x)) return UNFIX(x);
 
   if (Sbignump(x)) {
@@ -233,7 +233,7 @@ iptr S_integer_value(who, x) const char *who; ptr x; {
 }
 
 /* -2^(b-1) <= x <= 2^b-1, where b = number of bits in a uptr */
-IBOOL S_integer_valuep(x) ptr x; {
+IBOOL S_integer_valuep(ptr x) {
   if (Sfixnump(x)) return 1;
 
   if (Sbignump(x)) {
@@ -254,12 +254,12 @@ IBOOL S_integer_valuep(x) ptr x; {
   return 0;
 }
 
-iptr Sinteger_value(x) ptr x; {
+iptr Sinteger_value(ptr x) {
   return S_integer_value("Sinteger_value", x);
 }
 
 /* -2^31 <= x <= 2^32-1 */
-I32 S_int32_value(who, x) char *who; ptr x; {
+I32 S_int32_value(char *who, ptr x) {
 #if (fixnum_bits > 32)
   if (Sfixnump(x)) {
     iptr n = UNFIX(x);
@@ -299,12 +299,12 @@ I32 S_int32_value(who, x) char *who; ptr x; {
   return 0 /* not reached */;
 }
 
-I32 Sinteger32_value(x) ptr x; {
+I32 Sinteger32_value(ptr x) {
   return S_int32_value("Sinteger32_value", x);
 }
 
 /* -2^63 <= x <= 2^64-1 */
-I64 S_int64_value(who, x) char *who; ptr x; {
+I64 S_int64_value(char *who, ptr x) {
   if (Sfixnump(x)) return UNFIX(x);
 
   if (Sbignump(x)) {
@@ -329,11 +329,11 @@ I64 S_int64_value(who, x) char *who; ptr x; {
   return 0 /* not reached */;
 }
 
-I64 Sinteger64_value(x) ptr x; {
+I64 Sinteger64_value(ptr x) {
   return S_int64_value("Sinteger64_value", x);
 }
 
-ptr Sunsigned(u) uptr u; { /* convert arg to Scheme integer */
+ptr Sunsigned(uptr u) { /* convert arg to Scheme integer */
   if (u <= most_positive_fixnum)
     return FIX(u);
   else {
@@ -344,7 +344,7 @@ ptr Sunsigned(u) uptr u; { /* convert arg to Scheme integer */
   }
 }
 
-ptr Sinteger(i) iptr i; { /* convert arg to Scheme integer */
+ptr Sinteger(iptr i) { /* convert arg to Scheme integer */
   if (FIXRANGE(i))
     return FIX(i);
   else {
@@ -355,7 +355,7 @@ ptr Sinteger(i) iptr i; { /* convert arg to Scheme integer */
   }
 }
 
-ptr Sunsigned32(u) U32 u; { /* convert arg to Scheme integer */
+ptr Sunsigned32(U32 u) { /* convert arg to Scheme integer */
 #if (fixnum_bits > 32)
   return FIX((uptr)u);
 #else
@@ -370,7 +370,7 @@ ptr Sunsigned32(u) U32 u; { /* convert arg to Scheme integer */
 #endif
 }
 
-ptr Sinteger32(i) I32 i; { /* convert arg to Scheme integer */
+ptr Sinteger32(I32 i) { /* convert arg to Scheme integer */
 #if (fixnum_bits > 32)
   return FIX((iptr)i);
 #else
@@ -385,7 +385,7 @@ ptr Sinteger32(i) I32 i; { /* convert arg to Scheme integer */
 #endif
 }
 
-ptr Sunsigned64(u) U64 u; { /* convert arg to Scheme integer */
+ptr Sunsigned64(U64 u) { /* convert arg to Scheme integer */
   if (u <= most_positive_fixnum)
     return FIX((uptr)u);
   else {
@@ -396,7 +396,7 @@ ptr Sunsigned64(u) U64 u; { /* convert arg to Scheme integer */
   }
 }
 
-ptr Sinteger64(i) I64 i; { /* convert arg to Scheme integer */
+ptr Sinteger64(I64 i) { /* convert arg to Scheme integer */
   if (i > most_negative_fixnum && i <= most_positive_fixnum)
     return FIX((iptr)i);
   else {
@@ -462,7 +462,7 @@ comparison
 ***
 */
 
-IBOOL S_big_lt(x, y) ptr x, y; {
+IBOOL S_big_lt(ptr x, ptr y) {
   if (BIGSIGN(x))
     if (BIGSIGN(y))
       return abs_big_lt(y, x, BIGLEN(y), BIGLEN(x)); /* both negative */
@@ -475,11 +475,11 @@ IBOOL S_big_lt(x, y) ptr x, y; {
       return abs_big_lt(x, y, BIGLEN(x), BIGLEN(y)); /* both positive */
 }
 
-IBOOL S_big_eq(x, y) ptr x; ptr y; {
+IBOOL S_big_eq(ptr x, ptr y) {
   return (BIGSIGN(x) == BIGSIGN(y)) && abs_big_eq(x, y, BIGLEN(x), BIGLEN(y));
 }
 
-static IBOOL abs_big_lt(x, y, xl, yl) ptr x, y; iptr xl, yl; {
+static IBOOL abs_big_lt(ptr x, ptr y, iptr xl, iptr yl) {
   if (xl != yl)
     return xl < yl;
   else {
@@ -492,7 +492,7 @@ static IBOOL abs_big_lt(x, y, xl, yl) ptr x, y; iptr xl, yl; {
   }
 }
 
-static IBOOL abs_big_eq(x, y, xl, yl) ptr x, y; iptr xl, yl; {
+static IBOOL abs_big_eq(ptr x, ptr y, iptr xl, iptr yl) {
   if (xl != yl)
     return 0;
   else {
@@ -511,16 +511,16 @@ addition/subtraction
 ***
 */
 
-static ptr big_negate(tc, x) ptr tc, x; {
+static ptr big_negate(ptr tc, ptr x) {
   return copy_normalize(tc, &BIGIT(x,0),BIGLEN(x),!BIGSIGN(x));
 }
 
-ptr S_big_negate(x) ptr x; {
+ptr S_big_negate(ptr x) {
   return big_negate(get_thread_context(), x);
 }
 
 /* assumptions: BIGLEN(x) >= BIGLEN(y) */
-static ptr big_add_pos(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL sign; {
+static ptr big_add_pos(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign) {
   iptr i;
   bigit *xp, *yp, *zp;
   bigit k = 0;
@@ -542,7 +542,7 @@ static ptr big_add_pos(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL
 }
 
 /* assumptions: x >= y */
-static ptr big_add_neg(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL sign; {
+static ptr big_add_neg(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign) {
   iptr i;
   bigit *xp, *yp, *zp;
   bigit b = 0;
@@ -561,7 +561,7 @@ static ptr big_add_neg(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL
   return copy_normalize(tc, zp+1,xl,sign);
 }
 
-static ptr big_add(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL xs, ys; {
+static ptr big_add(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys) {
   if (xs == ys)
     if (xl < yl)
       return big_add_pos(tc, y, x, yl, xl, xs);
@@ -575,7 +575,7 @@ static ptr big_add(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL xs
 }
 
 /* arguments must be integers, fixnums or bignums */
-ptr S_add(x, y) ptr x, y; {
+ptr S_add(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   if (Sfixnump(x)) {
@@ -599,7 +599,7 @@ ptr S_add(x, y) ptr x, y; {
 }
 
 /* arguments must be integers, fixnums or bignums */
-ptr S_sub(x, y) ptr x, y; {
+ptr S_sub(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   if (Sfixnump(x)) {
@@ -628,7 +628,7 @@ multiplication
 ***
 */
 
-static ptr big_mul(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL sign; {
+static ptr big_mul(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign) {
   iptr xi, yi;
   bigit *xp, *yp, *zp, *zpa;
   bigit k, k1, prod;
@@ -662,7 +662,7 @@ static ptr big_mul(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL sign
 #define SHORTRANGE(x) (-0x3FFFFFFF <= (x) && (x) <= 0x3FFFFFFF)
 #endif
 
-ptr S_mul(x, y) ptr x, y; {
+ptr S_mul(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   iptr xl, yl; IBOOL xs, ys;
@@ -700,7 +700,7 @@ division
 */
 
 /* arguments must be integers (fixnums or bignums), y must be nonzero */
-ptr S_div(x, y) ptr x, y; {
+ptr S_div(ptr x, ptr y) {
   ptr g, n, d;
   ptr tc = get_thread_context();
 
@@ -715,20 +715,20 @@ ptr S_div(x, y) ptr x, y; {
   return S_rational(n, d);
 }
 
-ptr S_trunc(x, y) ptr x, y; {
+ptr S_trunc(ptr x, ptr y) {
   ptr q;
   S_trunc_rem(get_thread_context(), x, y, &q, (ptr *)NULL);
   return q;
 }
 
-ptr S_rem(x, y) ptr x, y; {
+ptr S_rem(ptr x, ptr y) {
   ptr r;
   S_trunc_rem(get_thread_context(), x, y, (ptr *)NULL, &r);
   return r;
 }
 
 /* arguments must be integers (fixnums or bignums), y must be nonzero */
-void S_trunc_rem(tc, origx, y, q, r) ptr tc, origx, y, *q, *r; {
+void S_trunc_rem(ptr tc, ptr origx, ptr y, ptr *q, ptr *r) {
   iptr xl, yl; IBOOL xs, ys; ptr x = origx;
 
   if (Sfixnump(x)) {
@@ -781,8 +781,7 @@ static void big_short_trunc(ptr tc, ptr x, bigit s, iptr xl, IBOOL qs, IBOOL rs,
   if (r != (ptr *)NULL) *r = copy_normalize(tc, &k,1,rs);
 }
 
-static void big_trunc(tc, x, y, xl, yl, qs, rs, q, r)
-              ptr tc, x, y; iptr xl, yl; IBOOL qs, rs; ptr *q, *r; {
+static void big_trunc(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL qs, IBOOL rs, ptr *q, ptr *r) {
   iptr i;
   bigit *p, *xp, *yp;
   iptr m = xl-yl+1;
@@ -816,7 +815,7 @@ static void big_trunc(tc, x, y, xl, yl, qs, rs, q, r)
   }
 }
 
-static INT normalize(xp, yp, xl, yl) bigit *xp, *yp; iptr xl, yl; {
+static INT normalize(bigit *xp, bigit *yp, iptr xl, iptr yl) {
   iptr i;
   bigit *p, k, b;
   INT shft;
@@ -832,7 +831,7 @@ static INT normalize(xp, yp, xl, yl) bigit *xp, *yp; iptr xl, yl; {
   return shft;
 }
 
-static bigit quotient_digit(tc, xp, yp, yl) ptr tc; bigit *xp, *yp; iptr yl; {
+static bigit quotient_digit(ptr tc, bigit *xp, bigit *yp, iptr yl) {
   bigit *p1, *p2, q, k, b, prod;
   iptr i;
 
@@ -859,7 +858,7 @@ static bigit quotient_digit(tc, xp, yp, yl) ptr tc; bigit *xp, *yp; iptr yl; {
   return q;
 }
 
-static bigit qhat(xp, yp) bigit *xp, *yp; {
+static bigit qhat(bigit *xp, bigit *yp) {
   bigit q, r, high, low, k;
 
   k = 0;
@@ -886,7 +885,7 @@ gcd
 ***
 */
 
-static ptr uptr_gcd(x, y) uptr x, y; {
+static ptr uptr_gcd(uptr x, uptr y) {
   uptr r;
 
   while (y != 0) {
@@ -906,13 +905,15 @@ static ptr big_short_gcd(ptr tc, ptr x, bigit y, iptr xl) {
 
   if (y == 0) return BIGSIGN(x) ? big_negate(tc, x) : x;
 
+  USE_TRAP_FUEL(tc, xl);
+
   for (i = xl, r = 0, xp = &BIGIT(x,0); i-- > 0; )
     EDIV(r, *xp++, y, &q, &r)
 
   return uptr_gcd((uptr)y,(uptr)r);
 }
 
-static ptr big_gcd(tc, x, y, xl, yl) ptr tc, x, y; iptr xl, yl; {
+static ptr big_gcd(ptr tc, ptr x, ptr y, iptr xl, iptr yl) {
   iptr i;
   INT shft, asc;
   bigit *p, *xp, *yp, k, b;
@@ -991,7 +992,7 @@ static ptr big_gcd(tc, x, y, xl, yl) ptr tc, x, y; iptr xl, yl; {
   }
 }
 
-ptr S_gcd(x, y) ptr x, y; {
+ptr S_gcd(ptr x, ptr y) {
   ptr tc = get_thread_context();
   iptr xl, yl; IBOOL xs, ys;
 
@@ -1040,15 +1041,15 @@ floating-point operations
 
 #ifdef IEEE_DOUBLE
 /* exponent stored + 1024, hidden bit to left of decimal point */
-#define bias 1023
-#define bitstoright 52
-#define m1mask 0xf
-#ifdef WIN32
-#define hidden_bit 0x10000000000000
-#else
-#define hidden_bit 0x10000000000000ULL
-#endif
-#ifdef LITTLE_ENDIAN_IEEE_DOUBLE
+# define bias 1023
+# define bitstoright 52
+# define m1mask 0xf
+# ifdef WIN32
+#  define hidden_bit 0x10000000000000
+# else
+#  define hidden_bit 0x10000000000000ULL
+# endif
+# ifdef LITTLE_ENDIAN_IEEE_DOUBLE
 struct dblflt {
     UINT m4: 16;
     UINT m3: 16;
@@ -1057,7 +1058,7 @@ struct dblflt {
     UINT e: 11;
     UINT sign: 1;
 };
-#else
+# else
 struct dblflt {
     UINT sign: 1;
     UINT e: 11;
@@ -1066,10 +1067,10 @@ struct dblflt {
     UINT m3: 16;
     UINT m4: 16;
 };
-#endif
+# endif
 #endif
 
-double S_random_double(m1, m2, m3, m4, scale) U32 m1, m2, m3, m4; double scale; {
+double S_random_double(U32 m1, U32 m2, U32 m3, U32 m4, double scale) {
  /* helper for s_fldouble in prim5.c */
   union dxunion {
     double d;
@@ -1109,7 +1110,7 @@ static double big_short_floatify(ptr tc, ptr x, bigit s, iptr xl, IBOOL sign) {
   return floatify_normalize(&BIGIT(W(tc),0), xl*bigit_bits, sign, k != 0);
 }
 
-static double big_floatify(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IBOOL sign; {
+static double big_floatify(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL sign) {
   iptr i, ul;
   bigit *p, *xp, *yp, k;
 
@@ -1140,7 +1141,7 @@ static double big_floatify(tc, x, y, xl, yl, sign) ptr tc, x, y; iptr xl, yl; IB
 }
 
 /* come in with exactly 'enough' bigits */
-static double floatify_normalize(p, e, sign, sticky) bigit *p; iptr e; IBOOL sign, sticky; {
+static double floatify_normalize(bigit *p, iptr e, IBOOL sign, IBOOL sticky) {
   /* *p: first bigit; e: exponent; sign: sign; sticky: sticky bit */
   union dxunion {
     double d;
@@ -1220,7 +1221,7 @@ static double floatify_normalize(p, e, sign, sticky) bigit *p; iptr e; IBOOL sig
   return dx.d;
 }
 
-static double floatify_ratnum(tc, p) ptr tc, p; {
+static double floatify_ratnum(ptr tc, ptr p) {
   ptr x, y; iptr xl, yl; IBOOL xs;
 
   x = RATNUM(p); y = RATDEN(p);
@@ -1252,7 +1253,7 @@ static double floatify_ratnum(tc, p) ptr tc, p; {
     return big_floatify(tc, x, y, xl, yl, xs);
 }
 
-double S_floatify(x) ptr x; {
+double S_floatify(ptr x) {
   ptr tc = get_thread_context();
 
   if (Sflonump(x)) return FLODAT(x);
@@ -1265,7 +1266,7 @@ double S_floatify(x) ptr x; {
 }
 
 #ifdef IEEE_DOUBLE
-ptr S_decode_float(d) double d; {
+ptr S_decode_float(double d) {
   union dxunion {
     double d;
     struct dblflt x;
@@ -1311,7 +1312,7 @@ logical operations
 ***
 */
 
-static ptr s_big_ash(tc, xp, xl, sign, cnt) ptr tc; bigit *xp; iptr xl; IBOOL sign; iptr cnt; {
+static ptr s_big_ash(ptr tc, bigit *xp, iptr xl, IBOOL sign, iptr cnt) {
   iptr i;
   bigit *p1, *p2, k;
 
@@ -1396,7 +1397,7 @@ static ptr s_big_ash(tc, xp, xl, sign, cnt) ptr tc; bigit *xp; iptr xl; IBOOL si
 }
 
 /* x is a bignum or fixnum, n is a fixnum */
-ptr S_ash(x, n) ptr x, n; {
+ptr S_ash(ptr x, ptr n) {
   ptr tc = get_thread_context();
   iptr cnt = UNFIX(n);
 
@@ -1413,7 +1414,7 @@ ptr S_ash(x, n) ptr x, n; {
 }
 
 /* x is a bignum */
-ptr S_integer_length(x) ptr x; {
+ptr S_integer_length(ptr x) {
   iptr a; bigit b;
 
   if (BIGSIGN(x)) x = S_sub(FIX(-1), x);
@@ -1491,7 +1492,8 @@ ptr S_big_positive_bit_field(ptr x, ptr fxstart, ptr fxend) {
 }
 
 /* returns a lower bound on the number of trailing 0 bits in the
-   binary representation: */
+   binary representation; the result plus bigit_bits-1 is an
+   upper bound: */
 ptr S_big_trailing_zero_bits(ptr x) {
   bigit *xp = &BIGIT(x, 0);
   iptr xl = BIGLEN(x), i;
@@ -1524,7 +1526,7 @@ ptr S_big_trailing_zero_bits(ptr x) {
    # = 2's complement; #x = ~x + 1 = ~(x - 1) if x > 0
 */
 
-ptr S_logand(x, y) ptr x, y; {
+ptr S_logand(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   if (Sfixnump(x)) {
@@ -1561,7 +1563,7 @@ ptr S_logand(x, y) ptr x, y; {
 */
 
 /* assumes xl >= yl */
-static ptr big_logand(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL xs, ys; {
+static ptr big_logand(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys) {
   iptr i;
   bigit *xp, *yp, *zp;
 
@@ -1629,7 +1631,7 @@ static ptr big_logand(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL
 
 /* logtest is like logand but returns a boolean value */
 
-ptr S_logtest(x, y) ptr x, y; {
+ptr S_logtest(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   if (Sfixnump(x)) {
@@ -1658,7 +1660,7 @@ ptr S_logtest(x, y) ptr x, y; {
    logand would return a nonzero value */
 
 /* assumes xl >= yl */
-static ptr big_logtest(x, y, xl, yl, xs, ys) ptr x, y; iptr xl, yl; IBOOL xs, ys; {
+static ptr big_logtest(ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys) {
   iptr i;
   bigit *xp, *yp;
 
@@ -1700,7 +1702,7 @@ static ptr big_logtest(x, y, xl, yl, xs, ys) ptr x, y; iptr xl, yl; IBOOL xs, ys
 }
 
 /* k must be a nonnegative fixnum.  x may be a bignum or fixnum */
-ptr S_logbitp(k, x) ptr k, x; {
+ptr S_logbitp(ptr k, ptr x) {
   uptr n = UNFIX(k);
 
   if (Sfixnump(x)) {
@@ -1724,7 +1726,7 @@ void S_bignum_mask_test(void) {
 
 /* similar logic to big_logand */
 
-static ptr big_logbitp(n, x, xl, xs) ptr x; iptr n, xl; IBOOL xs; {
+static ptr big_logbitp(iptr n, ptr x, iptr xl, IBOOL xs) {
   iptr i;
   bigit *xp;
 
@@ -1751,7 +1753,7 @@ static ptr big_logbitp(n, x, xl, xs) ptr x; iptr n, xl; IBOOL xs; {
 }
 
 /* k must be a nonnegative fixnum.  x may be a bignum or fixnum */
-ptr S_logbit0(k, x) ptr k, x; {
+ptr S_logbit0(ptr k, ptr x) {
   ptr tc = get_thread_context();
   iptr n = UNFIX(k);
 
@@ -1778,7 +1780,7 @@ ptr S_logbit0(k, x) ptr k, x; {
 */
 
 /* adapted from big_logor algorithm */
-static ptr big_logbit0(tc, origx, n, x, xl, xs) ptr tc, origx, x; iptr n, xl; IBOOL xs; {
+static ptr big_logbit0(ptr tc, ptr origx, iptr n, ptr x, iptr xl, IBOOL xs) {
   iptr i;
   bigit *xp, *zp;
   iptr yl = (n / bigit_bits) + 1;
@@ -1828,7 +1830,7 @@ static ptr big_logbit0(tc, origx, n, x, xl, xs) ptr tc, origx, x; iptr n, xl; IB
 }
 
 /* k must be a nonnegative fixnum.  x may be a bignum or fixnum */
-ptr S_logbit1(k, x) ptr k, x; {
+ptr S_logbit1(ptr k, ptr x) {
   ptr tc = get_thread_context();
   iptr n = UNFIX(k);
 
@@ -1847,7 +1849,7 @@ ptr S_logbit1(k, x) ptr k, x; {
 }
 
 /* adapted from big_logor algorithm */
-static ptr big_logbit1(tc, origx, n, x, xl, xs) ptr tc, origx, x; iptr n, xl; IBOOL xs; {
+static ptr big_logbit1(ptr tc, ptr origx, iptr n, ptr x, iptr xl, IBOOL xs) {
   iptr i;
   bigit *xp, *zp;
   iptr yl = (n / bigit_bits) + 1;
@@ -1901,7 +1903,7 @@ static ptr big_logbit1(tc, origx, n, x, xl, xs) ptr tc, origx, x; iptr n, xl; IB
   }
 }
 
-ptr S_logor(x, y) ptr x, y; {
+ptr S_logor(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   if (Sfixnump(x)) {
@@ -1938,7 +1940,7 @@ ptr S_logor(x, y) ptr x, y; {
 */
 
 /* assumes xl >= yl */
-static ptr big_logor(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL xs, ys; {
+static ptr big_logor(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys) {
   iptr i;
   bigit *xp, *yp, *zp;
 
@@ -2008,7 +2010,7 @@ static ptr big_logor(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL
   }
 }
 
-ptr S_logxor(x, y) ptr x, y; {
+ptr S_logxor(ptr x, ptr y) {
   ptr tc = get_thread_context();
 
   if (Sfixnump(x)) {
@@ -2045,7 +2047,7 @@ ptr S_logxor(x, y) ptr x, y; {
 */
 
 /* assumes xl >= yl */
-static ptr big_logxor(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL xs, ys; {
+static ptr big_logxor(ptr tc, ptr x, ptr y, iptr xl, iptr yl, IBOOL xs, IBOOL ys) {
   iptr i;
   bigit *xp, *yp, *zp;
 
@@ -2139,7 +2141,7 @@ static ptr big_logxor(tc, x, y, xl, yl, xs, ys) ptr tc, x, y; iptr xl, yl; IBOOL
      (define (lognot x) (- -1 x))
 */
 
-ptr S_lognot(x) ptr x; {
+ptr S_lognot(ptr x) {
   if (Sfixnump(x)) {
     return FIX(~UNFIX(x));
   } else {
@@ -2147,7 +2149,7 @@ ptr S_lognot(x) ptr x; {
   }
 }
 
-void S_number_init() {
+void S_number_init(void) {
   if ((int)(hidden_bit >> 22) != 0x40000000) {
     fprintf(stderr, "hidden_bit >> 22 = %x\n", (int)(hidden_bit >> 22));
     S_abnormal_exit();
