@@ -20,7 +20,6 @@
 #include <limits.h>
 #ifdef WIN32
 #include <io.h>
-#include <time.h>
 #else
 #include <sys/time.h>
 #endif
@@ -41,13 +40,13 @@ static heap_state current_state = UNINITIALIZED;
 /* INITIALIZATION SUPPORT */
 
 /* locally defined functions */
-static void main_init PROTO((void));
-static void idiot_checks PROTO((void));
-static INT run_script PROTO((const char *who, const char *scriptfile, INT argc, const char *argv[], IBOOL programp));
+static void main_init(void);
+static void idiot_checks(void);
+static INT run_script(const char *who, const char *scriptfile, INT argc, const char *argv[], IBOOL programp);
 
 extern void scheme_include(void);
 
-static void main_init() {
+static void main_init(void) {
     ptr tc = get_thread_context();
     ptr p;
     INT i;
@@ -104,16 +103,12 @@ static void main_init() {
                  i & 0x10 ? 4 : i & 0x20 ? 5 : i & 0x40 ? 6 : i & 0x80 ? 7 : 0);
     }
 
-    NULLIMMUTABLEVECTOR(tc) = S_G.null_immutable_vector;
-    NULLIMMUTABLEFXVECTOR(tc) = S_G.null_immutable_fxvector;
-    NULLIMMUTABLEBYTEVECTOR(tc) = S_G.null_immutable_bytevector;
-    NULLIMMUTABLESTRING(tc) = S_G.null_immutable_string;
-
     PARAMETERS(tc) = S_G.null_vector;
     for (i = 0 ; i < virtual_register_count ; i += 1) {
       VIRTREG(tc, i) = FIX(0);
     }
 
+    S_thread_start_code_write(tc, 0, 0, NULL, 0);
     p = S_code(tc, type_code, size_rp_header);
     CODERELOC(p) = S_relocation_table(0);
     CODENAME(p) = Sfalse;
@@ -127,6 +122,7 @@ static void main_init() {
         (uptr)TO_PTR(&RPHEADERTOPLINK(TO_PTR(&CODEIT(p, 0)))) - (uptr)p;
     S_protect(&S_G.dummy_code_object);
     S_G.dummy_code_object = p;
+    S_thread_end_code_write(tc, 0, 0, NULL, 0);
 
     S_protect(&S_G.error_invoke_code_object);
     S_G.error_invoke_code_object = Snil;
@@ -151,14 +147,16 @@ static void main_init() {
 
 static ptr fixtest = FIX(-1);
 
-static void idiot_checks() {
+static void idiot_checks(void) {
   IBOOL oops = 0;
 
-  if (bytes_per_segment < S_pagesize) {
+#ifndef PORTABLE_BYTECODE
+  if (minimum_segment_request * bytes_per_segment < S_pagesize) {
     fprintf(stderr, "bytes_per_segment (%x) < S_pagesize (%lx)\n",
               bytes_per_segment, (long)S_pagesize);
     oops = 1;
   }
+#endif
   if (sizeof(iptr) != sizeof(ptr)) {
     fprintf(stderr, "sizeof(iptr) [%ld] != sizeof(ptr) [%ld]\n",
               (long)sizeof(iptr), (long)sizeof(ptr));
@@ -198,12 +196,12 @@ static void idiot_checks() {
   }
 #endif
 #endif
+#ifndef PORTABLE_BYTECODE
   if (sizeof(wchar_t) * 8 != wchar_bits) {
     fprintf(stderr, "sizeof(wchar_t) * 8 [%ld] != wchar_bits [%d]\n",
               (long)sizeof(wchar_t), wchar_bits);
     oops = 1;
   }
-#ifndef PORTABLE_BYTECODE
   if (sizeof(size_t) * 8 != size_t_bits) {
     fprintf(stderr, "sizeof(size_t) * 8 [%ld] != size_t_bits [%d]\n",
               (long)sizeof(size_t), size_t_bits);
@@ -277,6 +275,14 @@ static void idiot_checks() {
     fprintf(stderr, "sizeof(string_char) [%ld] != string_char_bytes [%d]\n", (long)sizeof(string_char), string_char_bytes);
     oops = 1;
   }
+  if (TYPE((ptr)0, type_untyped) != (ptr)0) {
+    fprintf(stderr, "tagging with type_untyped changes an address\n");
+    oops = 1;
+  }
+  if (record_ptr_offset != record_type_disp) {
+    fprintf(stderr, "record_ptr_offset != record_type_disp\n");
+    oops = 1;
+  }
   if (UNFIX(fixtest) != -1) {
     fprintf(stderr, "UNFIX operation failed\n");
     oops = 1;
@@ -335,7 +341,24 @@ static void idiot_checks() {
     /* parallel GC relies on not confusing a forward marker with code flags */
     fprintf(stderr, "code flags overlap with forwadr_marker\n");
     oops = 1;
-  } 
+  }
+
+  if ((reference_disp != bytevector_data_disp)
+      || (reference_disp != flvector_data_disp)) {
+    fprintf(stderr, "reference displacement does not match bytevector or flvector displacement\n");
+    oops = 1;
+  }
+  
+  if (reference_disp >= (allocation_segment_tail_padding
+                         /* to determine the minimum distince from the start of an
+                            alocated object to the end of its alloted space, take the
+                            smaller of the allocation alignment or sizeof(double), where
+                            the latter is relevant for a flonum that points into the
+                            imaginary half of an inexactnum */
+                         + ((byte_alignment < sizeof(double)) ? byte_alignment : sizeof(double)))) {
+    fprintf(stderr, "reference displacement can extend past the end of an allocation page\n");
+    oops = 1;
+  }
 
   if (oops) S_abnormal_exit();
 }
@@ -344,11 +367,11 @@ static void idiot_checks() {
 /* SUPPORT FOR CALLING INTO SCHEME */
 
 /* locally defined functions */
-static ptr boot_call PROTO((ptr tc, ptr p, INT n));
-static void check_ap PROTO((ptr tc));
+static ptr boot_call(ptr tc, ptr p, INT n);
+static void check_ap(ptr tc);
 
 /* arguments and ac0 set up */
-static ptr boot_call(tc, p, n) ptr tc; ptr p; INT n; {
+static ptr boot_call(ptr tc, ptr p, INT n) {
     AC1(tc) = p;
     CP(tc) = Svoid; /* don't have calling code object */
 
@@ -372,7 +395,7 @@ static ptr boot_call(tc, p, n) ptr tc; ptr p; INT n; {
     return p;
 }
 
-static void check_ap(tc) ptr tc; {
+static void check_ap(ptr tc) {
     if ((uptr)AP(tc) & (byte_alignment - 1)) {
         (void) fprintf(stderr, "ap is not double word aligned\n");
         S_abnormal_exit();
@@ -383,7 +406,7 @@ static void check_ap(tc) ptr tc; {
     }
 }
 
-void S_generic_invoke(tc, code) ptr tc; ptr code; {
+void S_generic_invoke(ptr tc, ptr code) {
 #if defined(PORTABLE_BYTECODE)
   S_pb_interp(tc, (void *)&CODEIT(code,0));
 #elif defined(PPCAIX)
@@ -391,7 +414,7 @@ void S_generic_invoke(tc, code) ptr tc; ptr code; {
     hdr.entry = (caddr_t)&CODEIT(code,0);
     hdr.toc = (caddr_t)0;
     hdr.static_link = (caddr_t)0;
-    (*((void (*) PROTO((ptr)))(void *)&hdr))(tc);
+    (*((void (*)(ptr))(void *)&hdr))(tc);
 #elif defined(PPCNT)
   /* under NT, function headers contain no static link */
     struct {I32 entry, toc;} hdr;
@@ -412,18 +435,18 @@ void S_generic_invoke(tc, code) ptr tc; ptr code; {
     p(tc);
 #elif defined(WIN32) && !defined(__MINGW32__)
     __try {
-      (*((void (*) PROTO((ptr)))(void *)&CODEIT(code,0)))(tc);
+      (*((void (*)(ptr))(void *)&CODEIT(code,0)))(tc);
     }
     __except(GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION ?
              EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
     {
-        if (S_pants_down)
+        if (THREAD_GC(tc)->during_alloc)
             S_error_abort("nonrecoverable invalid memory reference");
         else
             S_error_reset("invalid memory reference");
     }
 #else
-    (*((void (*) PROTO((ptr)))(void *)&CODEIT(code,0)))(tc);
+    (*((void (*)(ptr))(void *)&CODEIT(code,0)))(tc);
 #endif
 }
 
@@ -431,11 +454,17 @@ void S_generic_invoke(tc, code) ptr tc; ptr code; {
 /* MISCELLANEOUS HELPERS */
 
 /* locally defined functions */
-static IBOOL next_path PROTO((char *path, const char *name, const char *ext, const char **sp, const char **dsp));
-static const char *path_last PROTO((const char *path));
-static char *get_defaultheapdirs PROTO((void));
+static IBOOL next_path(char *path, const char *name, const char *ext, const char **sp, const char **dsp);
+static const char *path_last(const char *path);
+static char *get_defaultheapdirs(void);
 
-static const char *path_last(p) const char *p; {
+#ifdef PATH_MAX
+# define BOOT_PATH_MAX PATH_MAX
+#else /* hack for Hurd: better to remove the restriction */
+# define BOOT_PATH_MAX 4096
+#endif
+
+static const char *path_last(const char *p) {
   const char *s;
 #ifdef WIN32
   char c;
@@ -460,7 +489,7 @@ static const char *path_last(p) const char *p; {
 
 static char *get_defaultheapdirs() {
   char *result;
-  wchar_t buf[PATH_MAX];
+  wchar_t buf[BOOT_PATH_MAX];
   DWORD len = sizeof(buf);
   if (ERROR_SUCCESS != RegGetValueW(HKEY_LOCAL_MACHINE, L"Software\\Chez Scheme\\csv" VERSION, L"HeapSearchPath", RRF_RT_REG_SZ, NULL, buf, &len))
     return DEFAULT_HEAP_PATH;
@@ -477,7 +506,11 @@ static char *get_defaultheapdirs() {
 #endif
 
 static char *get_defaultheapdirs() {
+#if defined(__EMSCRIPTEN__)
+  return ".";
+#else
   return DEFAULT_HEAP_PATH;
+#endif
 }
 #endif /* WIN32 */
 
@@ -485,13 +518,14 @@ static char *get_defaultheapdirs() {
  * leaving the full path with name affixed in path and *sp / *dsp pointing
  * past the current entry.  it returns 1 on success and 0 if at the end of
  * the search path.  path should be a pointer to an unoccupied buffer
- * PATH_MAX characters long.  either or both of sp/dsp may be empty,
+ * BOOT_PATH_MAX characters long.  either or both of sp/dsp may be empty,
  * but neither may be null, i.e., (char *)0. */
-static IBOOL next_path(path, name, ext, sp, dsp) char *path; const char *name, *ext, **sp, **dsp; {
+static IBOOL next_path(char *path, const char *name, const char *ext,
+                       const char **sp, const char **dsp) {
   char *p;
   const char *s, *t;
 
-#define setp(c) if (p >= path + PATH_MAX) { fprintf(stderr, "search path entry too long\n"); S_abnormal_exit(); } else *p++ = (c)
+#define setp(c) if (p >= path + BOOT_PATH_MAX) { fprintf(stderr, "search path entry too long\n"); S_abnormal_exit(); } else *p++ = (c)
   for (;;) {
     s = *sp;
     p = path;
@@ -504,10 +538,10 @@ static IBOOL next_path(path, name, ext, sp, dsp) char *path; const char *name, *
           switch (*s) {
 #ifdef WIN32
             case 'x': {
-              wchar_t exepath[PATH_MAX]; DWORD n;
+              wchar_t exepath[BOOT_PATH_MAX]; DWORD n;
               s += 1;
-              n = GetModuleFileNameW(NULL, exepath, PATH_MAX);
-              if (n == 0 || (n == PATH_MAX && GetLastError() == ERROR_INSUFFICIENT_BUFFER)) {
+              n = GetModuleFileNameW(NULL, exepath, BOOT_PATH_MAX);
+              if (n == 0 || (n == BOOT_PATH_MAX && GetLastError() == ERROR_INSUFFICIENT_BUFFER)) {
                 fprintf(stderr, "warning: executable path is too long; ignoring %%x\n");
               } else {
                 char *tstart;
@@ -577,29 +611,104 @@ static IBOOL next_path(path, name, ext, sp, dsp) char *path; const char *name, *
 
 typedef struct {
   INT fd;
-  char path[PATH_MAX];
+  iptr len; /* 0 => unknown */
+  iptr offset;
+  IBOOL need_check, close_after;
+  char path[BOOT_PATH_MAX];
 } boot_desc;
 
 #define MAX_BOOT_FILES 10
 static boot_desc bd[MAX_BOOT_FILES];
 
 /* locally defined functions */
-static octet get_u8 PROTO((INT fd));
-static uptr get_uptr PROTO((INT fd, uptr *pn));
-static INT get_string PROTO((INT fd, char *s, iptr max, INT *c));
-static IBOOL find_boot PROTO((const char *name, const char *ext, IBOOL direct_pathp, int fd, IBOOL errorp));
-static void load PROTO((ptr tc, iptr n, IBOOL base));
-static void check_boot_file_state PROTO((const char *who));
+static octet get_u8(INT fd);
+static uptr get_uptr(INT fd, uptr *pn);
+static INT get_string(INT fd, char *s, iptr max, INT *c);
+static void load(ptr tc, iptr n, IBOOL base);
+static void check_boot_file_state(const char *who);
 
-static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *ext; int fd; IBOOL direct_pathp, errorp; {
-  char pathbuf[PATH_MAX], buf[PATH_MAX];
+static IBOOL check_boot(int fd, IBOOL verbose, const char *path) {
+  uptr n = 0;
+
+  /* check for magic number */
+  if (get_u8(fd) != fasl_type_header ||
+      get_u8(fd) != 0 ||
+      get_u8(fd) != 0 ||
+      get_u8(fd) != 0 ||
+      get_u8(fd) != 'c' ||
+      get_u8(fd) != 'h' ||
+      get_u8(fd) != 'e' ||
+      get_u8(fd) != 'z') {
+    if (verbose) fprintf(stderr, "malformed fasl-object header in %s\n", path);
+    CLOSE(fd);
+    return 0;
+  }
+
+  /* check version */
+  if (get_uptr(fd, &n) != 0) {
+    if (verbose) fprintf(stderr, "unexpected end of file on %s\n", path);
+    CLOSE(fd);
+    return 0;
+  }
+
+  if (n != scheme_version) {
+    if (verbose) {
+      fprintf(stderr, "%s is for Version %s; ", path, S_format_scheme_version(n));
+      /* use separate fprintf since S_format_scheme_version returns static string */
+      fprintf(stderr, "need Version %s\n", S_format_scheme_version(scheme_version));
+    }
+    CLOSE(fd);
+    return 0;
+  }
+
+  /* check machine type */
+  if (get_uptr(fd, &n) != 0) {
+    if (verbose) fprintf(stderr, "unexpected end of file on %s\n", path);
+    CLOSE(fd);
+    return 0;
+  }
+
+  if (n != machine_type) {
+    if (verbose)
+      fprintf(stderr, "%s is for machine-type %s; need machine-type %s\n", path,
+              S_lookup_machine_type(n), S_lookup_machine_type(machine_type));
+    CLOSE(fd);
+    return 0;
+  }
+
+  return 1;
+}
+
+static void check_dependencies_header(int fd, const char *path) {
+  if (get_u8(fd) != '(') {  /* ) */
+    fprintf(stderr, "malformed boot file %s\n", path);
+    CLOSE(fd);
+    S_abnormal_exit();
+  }
+}
+
+static void finish_dependencies_header(int fd, const char *path, int c) {
+  while (c != ')') {
+    if (c < 0) {
+      fprintf(stderr, "malformed boot file %s\n", path);
+      CLOSE(fd);
+      S_abnormal_exit();
+    }
+    c = get_u8(fd);
+  }
+}
+
+static IBOOL find_boot(const char *name, const char *ext, IBOOL direct_pathp,
+                       int fd,
+                       IBOOL errorp) {
+  char pathbuf[BOOT_PATH_MAX], buf[BOOT_PATH_MAX];
   uptr n = 0;
   INT c;
   const char *path;
   char *expandedpath;
 
   if ((fd != -1) || direct_pathp || S_fixedpathp(name)) {
-    if (strlen(name) >= PATH_MAX) {
+    if (strlen(name) >= BOOT_PATH_MAX) {
       fprintf(stderr, "boot-file path is too long %s\n", name);
       S_abnormal_exit();
     }
@@ -623,53 +732,14 @@ static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *e
     }
     if (verbose) fprintf(stderr, "trying %s...opened\n", path);
 
-   /* check for magic number */
-    if (get_u8(fd) != fasl_type_header ||
-        get_u8(fd) != 0 ||
-        get_u8(fd) != 0 ||
-        get_u8(fd) != 0 ||
-        get_u8(fd) != 'c' ||
-        get_u8(fd) != 'h' ||
-        get_u8(fd) != 'e' ||
-        get_u8(fd) != 'z') {
-      fprintf(stderr, "malformed fasl-object header in %s\n", path);
+    if (!check_boot(fd, 1, path))
       S_abnormal_exit();
-    }
-
-   /* check version */
-    if (get_uptr(fd, &n) != 0) {
-      fprintf(stderr, "unexpected end of file on %s\n", path);
-      CLOSE(fd);
-      S_abnormal_exit();
-    }
-
-    if (n != scheme_version) {
-      fprintf(stderr, "%s is for Version %s; ", path, S_format_scheme_version(n));
-     /* use separate fprintf since S_format_scheme_version returns static string */
-      fprintf(stderr, "need Version %s\n", S_format_scheme_version(scheme_version));
-      CLOSE(fd);
-      S_abnormal_exit();
-    }
-
-   /* check machine type */
-    if (get_uptr(fd, &n) != 0) {
-      fprintf(stderr, "unexpected end of file on %s\n", path);
-      CLOSE(fd);
-      S_abnormal_exit();
-    }
-
-    if (n != machine_type) {
-      fprintf(stderr, "%s is for machine-type %s; need machine-type %s\n", path,
-              S_lookup_machine_type(n), S_lookup_machine_type(machine_type));
-      CLOSE(fd);
-      S_abnormal_exit();
-    }
   } else {
     const char *sp = Sschemeheapdirs;
     const char *dsp = Sdefaultheapdirs;
 
     path = pathbuf;
-    for (;;) {
+    while (1) {
       if (!next_path(pathbuf, name, ext, &sp, &dsp)) {
         if (errorp) {
           fprintf(stderr, "cannot find compatible boot file %s%s in search path:\n  \"%s%s\"\n",
@@ -692,63 +762,14 @@ static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *e
 
       if (verbose) fprintf(stderr, "trying %s...opened\n", path);
 
-     /* check for magic number */
-      if (get_u8(fd) != fasl_type_header ||
-          get_u8(fd) != 0 ||
-          get_u8(fd) != 0 ||
-          get_u8(fd) != 0 ||
-          get_u8(fd) != 'c' ||
-          get_u8(fd) != 'h' ||
-          get_u8(fd) != 'e' ||
-          get_u8(fd) != 'z') {
-        if (verbose) fprintf(stderr, "malformed fasl-object header in %s\n", path);
-        CLOSE(fd);
-        continue;
-      }
-
-     /* check version */
-      if (get_uptr(fd, &n) != 0) {
-        if (verbose) fprintf(stderr, "unexpected end of file on %s\n", path);
-        CLOSE(fd);
-        continue;
-      }
-
-      if (n != scheme_version) {
-        if (verbose) {
-          fprintf(stderr, "%s is for Version %s; ", path, S_format_scheme_version(n));
-         /* use separate fprintf since S_format_scheme_version returns static string */
-          fprintf(stderr, "need Version %s\n", S_format_scheme_version(scheme_version));
-        }
-        CLOSE(fd);
-        continue;
-      }
-
-     /* check machine type */
-      if (get_uptr(fd, &n) != 0) {
-        if (verbose) fprintf(stderr, "unexpected end of file on %s\n", path);
-        CLOSE(fd);
-        continue;
-      }
-
-      if (n != machine_type) {
-        if (verbose)
-          fprintf(stderr, "%s is for machine-type %s; need machine-type %s\n", path,
-                  S_lookup_machine_type(n), S_lookup_machine_type(machine_type));
-        CLOSE(fd);
-        continue;
-      }
-
-      break;
+      if (check_boot(fd, verbose, path))
+        break;
     }
   }
 
   if (verbose) fprintf(stderr, "version and machine type check\n");
 
-  if (get_u8(fd) != '(') {  /* ) */
-    fprintf(stderr, "malformed boot file %s\n", path);
-    CLOSE(fd);
-    S_abnormal_exit();
-  }
+  check_dependencies_header(fd, path);
 
   /* ( */
   if ((c = get_u8(fd)) == ')') {
@@ -761,7 +782,7 @@ static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *e
     if (boot_count == 0) {
       for (;;) {
        /* try to load heap or boot file this boot file requires */
-        if (get_string(fd, buf, PATH_MAX, &c) != 0) {
+        if (get_string(fd, buf, BOOT_PATH_MAX, &c) != 0) {
           fprintf(stderr, "unexpected end of file on %s\n", path);
           CLOSE(fd);
           S_abnormal_exit();
@@ -781,7 +802,7 @@ static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *e
           c = get_u8(fd);
           for (sep = " "; ; sep = "or ") {
             if (c == ')') break;
-            (void) get_string(fd, buf, PATH_MAX, &c);
+            (void) get_string(fd, buf, BOOT_PATH_MAX, &c);
             fprintf(stderr, "%s%s.boot ", sep, buf);
           }
           fprintf(stderr, "required by %s\n", path);
@@ -792,14 +813,7 @@ static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *e
     }
 
    /* skip to end of header */
-    while (c != ')') {
-      if (c < 0) {
-        fprintf(stderr, "malformed boot file %s\n", path);
-        CLOSE(fd);
-        S_abnormal_exit();
-      }
-      c = get_u8(fd);
-    }
+    finish_dependencies_header(fd, path, c);
   }
 
   if (boot_count >= MAX_BOOT_FILES) {
@@ -808,6 +822,10 @@ static IBOOL find_boot(name, ext, direct_pathp, fd, errorp) const char *name, *e
   }
 
   bd[boot_count].fd = fd;
+  bd[boot_count].offset = 0;
+  bd[boot_count].len = 0;
+  bd[boot_count].need_check = 0;
+  bd[boot_count].close_after = 1;
   strcpy(bd[boot_count].path, path);
   boot_count += 1;
 
@@ -837,7 +855,7 @@ static uptr get_uptr(INT fd, uptr *pn) {
   return 0;
 }
 
-static INT get_string(fd, s, max, c) INT fd; char *s; iptr max; INT *c; {
+static INT get_string(INT fd, char *s, iptr max, INT *c) {
   while (max-- > 0) {
     if (*c < 0) return -1;
     if (*c == ' ' || *c == ')') {
@@ -880,9 +898,19 @@ static void boot_element(ptr tc, ptr x, iptr n) {
   }
 }
 
-static void load(tc, n, base) ptr tc; iptr n; IBOOL base; {
+static void load(ptr tc, iptr n, IBOOL base) {
   ptr x; iptr i;
 
+  if (bd[n].need_check) {
+    if (LSEEK(bd[n].fd, bd[n].offset, SEEK_SET) != bd[n].offset) {
+      fprintf(stderr, "seek in boot file %s failed\n", bd[n].path);
+      S_abnormal_exit();
+    }
+    check_boot(bd[n].fd, 1, bd[n].path);
+    check_dependencies_header(bd[n].fd, bd[n].path);
+    finish_dependencies_header(bd[n].fd, bd[n].path, 0);
+  }
+
   if (base) {
     S_G.error_invoke_code_object = S_boot_read(bd[n].fd, bd[n].path);
     if (!Scodep(S_G.error_invoke_code_object)) {
@@ -919,7 +947,8 @@ static void load(tc, n, base) ptr tc; iptr n; IBOOL base; {
   }
 
   S_G.load_binary = Sfalse;
-  CLOSE(bd[n].fd);
+  if (bd[n].close_after)
+    CLOSE(bd[n].fd);
 }
 
 /***************************************************************************/
@@ -955,7 +984,7 @@ const char *Skernel_version(void) {
   return VERSION;
 }
 
-extern void Sset_verbose(v) INT v; {
+extern void Sset_verbose(INT v) {
   verbose = v;
 }
 
@@ -968,10 +997,10 @@ extern void Sretain_static_relocation(void) {
 #endif
 
 static void default_abnormal_exit(void) {
-  exit(1);
+  abort();
 }
 
-extern void Sscheme_init(abnormal_exit) void (*abnormal_exit) PROTO((void)); {
+extern void Sscheme_init(void (*abnormal_exit)(void)) {
   S_abnormal_exit_proc = abnormal_exit ? abnormal_exit : default_abnormal_exit;
   S_errors_to_console = 1;
 
@@ -979,7 +1008,7 @@ extern void Sscheme_init(abnormal_exit) void (*abnormal_exit) PROTO((void)); {
   S_pagesize = GETPAGESIZE();
 
   idiot_checks();
-
+  
   switch (current_state) {
     case RUNNING:
       fprintf(stderr, "error (Sscheme_init): call Sscheme_deinit first to terminate\n");
@@ -1045,30 +1074,55 @@ static void check_boot_file_state(const char *who) {
   }
 }
 
-extern void Sregister_boot_file(name) const char *name; {
+extern void Sregister_boot_file(const char *name) {
   check_boot_file_state("Sregister_boot_file");
   find_boot(name, "", 0, -1, 1);
 }
 
-extern void Sregister_boot_direct_file(name) const char *name; {
+extern void Sregister_boot_direct_file(const char *name) {
   check_boot_file_state("Sregister_boot_direct_file");
   find_boot(name, "", 1, -1, 1);
 }
 
-extern void Sregister_boot_file_fd(name, fd) const char *name; int fd; {
+extern void Sregister_boot_file_fd(const char *name, int fd) {
   check_boot_file_state("Sregister_boot_file_fd");
   find_boot(name, "", 1, fd, 1);
 }
 
+extern void Sregister_boot_file_fd_region(const char *name,
+                                          int fd,
+                                          iptr offset,
+                                          iptr len,
+                                          int close_after) {
+  check_boot_file_state("Sregister_boot_file_fd");
+
+  if (strlen(name) >= BOOT_PATH_MAX) {
+    fprintf(stderr, "boot-file path is too long %s\n", name);
+    S_abnormal_exit();
+  }
+
+  bd[boot_count].fd = fd;
+  bd[boot_count].offset = offset;
+  bd[boot_count].len = len;
+  bd[boot_count].need_check = 1;
+  bd[boot_count].close_after = close_after;
+  strcpy(bd[boot_count].path, name);
+  boot_count += 1;
+}
+
 extern void Sregister_heap_file(UNUSED const char *path) {
   fprintf(stderr, "Sregister_heap_file: saved heap files are not presently supported\n");
   S_abnormal_exit();
 }
 
-extern void Sbuild_heap(kernel, custom_init) const char *kernel; void (*custom_init) PROTO((void)); {
+extern void Sbuild_heap(const char *kernel, void (*custom_init)(void)) {
   ptr tc = Svoid; /* initialize to make gcc happy */
   ptr p;
 
+#if defined(ALWAYS_USE_BOOT_FILE)
+  kernel = ALWAYS_USE_BOOT_FILE;
+#endif
+
   switch (current_state) {
     case UNINITIALIZED:
     case DEINITIALIZED:
@@ -1093,14 +1147,14 @@ extern void Sbuild_heap(kernel, custom_init) const char *kernel; void (*custom_i
     }
 
     name = path_last(kernel);
-    if (strlen(name) >= PATH_MAX) {
+    if (strlen(name) >= BOOT_PATH_MAX) {
       fprintf(stderr, "executable name too long: %s\n", name);
       S_abnormal_exit();
     }
 
 #ifdef WIN32
     { /* strip off trailing .exe, if any */
-      static char buf[PATH_MAX];
+      static char buf[BOOT_PATH_MAX];
       iptr n;
 
       n = strlen(name) - 4;
@@ -1168,14 +1222,14 @@ extern void Sbuild_heap(kernel, custom_init) const char *kernel; void (*custom_i
   S_errors_to_console = 0;
 }
 
-extern void Senable_expeditor(history_file) const char *history_file; {
+extern void Senable_expeditor(const char *history_file) {
   Scall1(S_symbol_value(Sstring_to_symbol("$enable-expeditor")), Strue);
   if (history_file != (const char *)0)
     Scall1(S_symbol_value(Sstring_to_symbol("$expeditor-history-file")),
            Sstring_utf8(history_file, -1));
 }
 
-extern INT Sscheme_start(argc, argv) INT argc; const char *argv[]; {
+extern INT Sscheme_start(INT argc, const char *argv[]) {
   ptr tc = get_thread_context();
   ptr arglist, p; INT i;
 
@@ -1245,11 +1299,11 @@ static INT run_script(const char *who, const char *scriptfile, INT argc, const c
   return p == Svoid ? 0 : 1;
 }
 
-extern INT Sscheme_script(scriptfile, argc, argv) const char *scriptfile; INT argc; const char *argv[]; {
+extern INT Sscheme_script(const char *scriptfile, INT argc, const char *argv[]) {
   return run_script("Sscheme_script", scriptfile, argc, argv, 0);
 }
 
-extern INT Sscheme_program(programfile, argc, argv) const char *programfile; INT argc; const char *argv[]; {
+extern INT Sscheme_program(const char *programfile, INT argc, const char *argv[]) {
   return run_script("Sscheme_program", programfile, argc, argv, 1);
 }
 
@@ -1258,7 +1312,7 @@ extern void Ssave_heap(UNUSED const char *path, UNUSED INT level) {
   S_abnormal_exit();
 }
 
-extern void Sscheme_deinit() {
+extern void Sscheme_deinit(void) {
   ptr p, tc = get_thread_context();
 
   switch (current_state) {
