@@ -31,7 +31,7 @@ EXPORT int idid(int x) {
     return id(id(x));
 }
 
-EXPORT int ididid(x) int x; {
+EXPORT int ididid(int x) {
     return idid(id(x));
 }
 
@@ -53,8 +53,9 @@ EXPORT double float_id(double x) {
 
 #ifdef _WIN32
 #include <stdlib.h>
+#include <string.h>
 
-EXPORT int windows_strcpy(char *dst, char *src) {
+EXPORT char *windows_strcpy(char *dst, char *src) {
   return strcpy(dst, src);
 }
 
