@@ -96,6 +96,7 @@ MZ_EXTERN void scheme_init_os_thread(void);
 #define BIGNUM_CACHE_SIZE 16
 #define STACK_CACHE_SIZE 32
 #define NUM_MORE_CONSTANT_STXES 24
+#define CONFIG_CALLBACK_CACHE_LEN 4
 
   /* The number of cached scope sets should be a power of 2: */
 #define NUM_RECENT_SCOPE_SETS 8
@@ -362,7 +363,7 @@ typedef struct Thread_Local_Variables {
   void *on_atomic_timeout_data_;
   int atomic_timeout_auto_suspend_;
   int atomic_timeout_atomic_level_;
-  struct Scheme_Object *configuration_callback_cache_[2];
+  struct Scheme_Object *configuration_callback_cache_[CONFIG_CALLBACK_CACHE_LEN];
   struct FFI_Orig_Place_Call *cached_orig_place_todo_;
   struct Scheme_Hash_Table *ffi_lock_ht_;
   struct Scheme_Object *is_syntax_proc_;
