@@ -24,6 +24,8 @@ static Scheme_Object *eq_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *eqv_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *equal_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *equalish_prim (int argc, Scheme_Object *argv[]);
+static Scheme_Object *equal_always_prim (int argc, Scheme_Object *argv[]);
+static Scheme_Object *equal_always_recur_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *chaperone_p (int argc, Scheme_Object *argv[]);
 static Scheme_Object *impersonator_p (int argc, Scheme_Object *argv[]);
 static Scheme_Object *procedure_impersonator_star_p (int argc, Scheme_Object *argv[]);
@@ -38,7 +40,7 @@ typedef struct Equal_Info {
   Scheme_Object *recur;
   Scheme_Object *next, *next_next;
   Scheme_Object *insp;
-  intptr_t for_chaperone; /* 3 => for impersonator */
+  intptr_t mode; /* EQUAL_MODE_EQUAL, etc. */
 } Equal_Info;
 
 static int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql);
@@ -108,8 +110,15 @@ void scheme_init_bool (Scheme_Startup_Env *env)
   scheme_equal_proc = p;
   scheme_addto_prim_instance("equal?", scheme_equal_proc, env);
 
-  scheme_addto_prim_instance("equal?/recur", 
-                             scheme_make_prim_w_arity(equalish_prim, "equal?/recur", 3, 3), 
+  scheme_addto_prim_instance("equal?/recur",
+                             scheme_make_prim_w_arity(equalish_prim, "equal?/recur", 3, 3),
+                             env);
+
+  scheme_addto_prim_instance("equal-always?",
+                             scheme_make_prim_w_arity(equal_always_prim, "equal-always?", 2, 2),
+                             env);
+  scheme_addto_prim_instance("equal-always?/recur",
+                             scheme_make_prim_w_arity(equal_always_recur_prim, "equal-always?/recur", 3, 3),
                              env);
 
   p = scheme_make_immed_prim(chaperone_p, "chaperone?", 1, 1);
@@ -175,7 +184,7 @@ XFORM_NONGCING static void init_equal_info(Equal_Info *eql)
   eql->next = NULL;
   eql->next_next = NULL;
   eql->insp = NULL;
-  eql->for_chaperone = 0;
+  eql->mode = EQUAL_MODE_EQUAL;
 }
 
 static Scheme_Object *
@@ -201,6 +210,31 @@ equalish_prim (int argc, Scheme_Object *argv[])
   return (is_equal(argv[0], argv[1], &eql) ? scheme_true : scheme_false);
 }
 
+static Scheme_Object *
+equal_always_prim (int argc, Scheme_Object *argv[])
+{
+  Equal_Info eql;
+
+  init_equal_info(&eql);
+  eql.mode = 5; /* mode 5: 'equal-always? */
+
+  return (is_equal(argv[0], argv[1], &eql) ? scheme_true : scheme_false);
+}
+
+static Scheme_Object *
+equal_always_recur_prim (int argc, Scheme_Object *argv[])
+{
+  Equal_Info eql;
+
+  scheme_check_proc_arity("equal-always?/recur", 2, 2, argc, argv);
+
+  init_equal_info(&eql);
+  eql.next_next = argv[2];
+  eql.mode = 5; /* mode 5: 'equal-always? */
+
+  return (is_equal(argv[0], argv[1], &eql) ? scheme_true : scheme_false);
+}
+
 int scheme_eq (Scheme_Object *obj1, Scheme_Object *obj2)
 {
   return SAME_OBJ(obj1, obj2);
@@ -337,7 +371,14 @@ int scheme_eqv (Scheme_Object *obj1, Scheme_Object *obj2)
   return (is_eqv(obj1, obj2) > 0);
 }
 
-XFORM_NONGCING int is_fast_equal (Scheme_Object *obj1, Scheme_Object *obj2, int for_chaperone)
+/* for_chaperone_alw:
+     0: 'equal? or 'impersonator-of?
+     1: 'chaperone-of? or 'equal-always?
+   result:
+     -1: unknown
+     0: known false
+     1: known true */
+XFORM_NONGCING int is_fast_equal (Scheme_Object *obj1, Scheme_Object *obj2, int for_chaperone_alw)
 {
   Scheme_Type t1, t2;
   int cmp;
@@ -356,6 +397,7 @@ XFORM_NONGCING int is_fast_equal (Scheme_Object *obj1, Scheme_Object *obj2, int
  case scheme_flvector_type:
    {
      intptr_t l1, l2, i;
+     if (for_chaperone_alw) return -1;
      l1 = SCHEME_FLVEC_SIZE(obj1);
      l2 = SCHEME_FLVEC_SIZE(obj2);
      if (l1 == l2) {
@@ -372,6 +414,7 @@ XFORM_NONGCING int is_fast_equal (Scheme_Object *obj1, Scheme_Object *obj2, int
  case scheme_extflvector_type:
    {
      intptr_t l1, l2, i;
+     if (for_chaperone_alw) return -1;
      l1 = SCHEME_EXTFLVEC_SIZE(obj1);
      l2 = SCHEME_EXTFLVEC_SIZE(obj2);
      if (l1 == l2) {
@@ -390,7 +433,7 @@ XFORM_NONGCING int is_fast_equal (Scheme_Object *obj1, Scheme_Object *obj2, int
  case scheme_windows_path_type:
    {
      intptr_t l1, l2;
-     if (for_chaperone) return -1;
+     if (for_chaperone_alw) return -1;
      l1 = SCHEME_BYTE_STRTAG_VAL(obj1);
      l2 = SCHEME_BYTE_STRTAG_VAL(obj2);
      return ((l1 == l2)
@@ -399,7 +442,7 @@ XFORM_NONGCING int is_fast_equal (Scheme_Object *obj1, Scheme_Object *obj2, int
  case scheme_char_string_type:
    {
      intptr_t l1, l2;
-     if (for_chaperone) return -1;
+     if (for_chaperone_alw) return -1;
      l1 = SCHEME_CHAR_STRTAG_VAL(obj1);
      l2 = SCHEME_CHAR_STRTAG_VAL(obj2);
      return ((l1 == l2)
@@ -436,13 +479,34 @@ int scheme_equal (Scheme_Object *obj1, Scheme_Object *obj2) XFORM_ASSERT_NO_CONV
 {
   int v;
 
-  v = is_fast_equal(obj1, obj2, 0);
+  v = is_fast_equal(obj1, obj2, 0); /* 0: 'equal? or 'impersonator-of? */
   if (v > -1)
     return v;
 
   return is_slow_equal(obj1, obj2);
 }
 
+int is_slow_equal_always (Scheme_Object *obj1, Scheme_Object *obj2)
+{
+  Equal_Info eql;
+
+  init_equal_info(&eql);
+  eql.mode = 5; /* mode 5: 'equal-always? */
+
+  return is_equal(obj1, obj2, &eql);
+}
+
+int scheme_equal_always (Scheme_Object *obj1, Scheme_Object *obj2) XFORM_ASSERT_NO_CONVERSION
+{
+  int v;
+
+  v = is_fast_equal(obj1, obj2, 1); /* 1: 'chaperone-of? or 'equal-always? */
+  if (v > -1)
+    return v;
+
+  return is_slow_equal_always(obj1, obj2);
+}
+
 static Scheme_Object *union_find(Scheme_Object *obj1, Scheme_Hash_Table *ht)
 {
   Scheme_Object *v, *prev = obj1, *prev_prev = obj1;
@@ -562,23 +626,23 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
   }
 
  top_after_next:
-  cmp = is_fast_equal(obj1, obj2, eql->for_chaperone == 1);
-  if (cmp > -1)
+  cmp = is_fast_equal(obj1, obj2, (eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS));
+  if (cmp > -1) /* cmp 0 or 1: known */
     return cmp;
 
-  if (eql->for_chaperone 
+  if (((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_IMPERSONATOR_OF))
       && SCHEME_CHAPERONEP(obj2)
       && (!(SCHEME_CHAPERONE_FLAGS((Scheme_Chaperone *)obj2) & SCHEME_CHAPERONE_IS_IMPERSONATOR)
-          || (eql->for_chaperone > 1))
+          || (eql->mode == EQUAL_MODE_IMPERSONATOR_OF))
       && scheme_is_noninterposing_chaperone(obj2)) {
     obj2 = ((Scheme_Chaperone *)obj2)->prev;
     goto top_after_next;
   }
 
-  if (eql->for_chaperone 
+  if (((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_IMPERSONATOR_OF))
       && SCHEME_CHAPERONEP(obj1)
       && (!(SCHEME_CHAPERONE_FLAGS((Scheme_Chaperone *)obj1) & SCHEME_CHAPERONE_IS_IMPERSONATOR)
-          || (eql->for_chaperone > 1))) {
+          || (eql->mode == EQUAL_MODE_IMPERSONATOR_OF))) {
     /* `obj1` and `obj2` are not eq, otherwise is_fast_equal()
        would have returned true */
     if (SCHEME_CHAPERONEP(obj2)) {
@@ -608,7 +672,7 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
       obj2 = (Scheme_Object *)scheme_hash_tree_resolve_placeholder((Scheme_Hash_Tree *)obj2);
       goto top_after_next;
     }
-    if (!eql->for_chaperone) {
+    if ((eql->mode == EQUAL_MODE_EQUAL) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS)) {
       if (SCHEME_CHAPERONEP(obj1)) {
 	/* OPT only use prev for unsafe-chaperone-vector, use val otherwise */
         obj1 = ((Scheme_Chaperone *)obj1)->prev;
@@ -641,7 +705,7 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     case scheme_mutable_pair_type:
       {
 #   include "mzeqchk.inc"
-        if (eql->for_chaperone == 1)
+        if ((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
           return 0;
         if (union_check(obj1, obj2, eql))
           return 1;
@@ -656,20 +720,39 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     case scheme_fxvector_type:
       {
 #   include "mzeqchk.inc"
-        if ((eql->for_chaperone == 1) && (!SCHEME_IMMUTABLEP(obj1)
-                                          || !SCHEME_IMMUTABLEP(obj2)))
+        if ((((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS)))
+            && (!SCHEME_IMMUTABLEP(obj1) || !SCHEME_IMMUTABLEP(obj2)))
           return 0;
         if (union_check(obj1, obj2, eql))
           return 1;
         return vector_equal(obj1, orig_obj1, obj2, orig_obj2, eql);
       }
+    case scheme_stencil_vector_type:
+      {
+        Scheme_Stencil_Vector *sv1 = (Scheme_Stencil_Vector *)obj1;
+        Scheme_Stencil_Vector *sv2 = (Scheme_Stencil_Vector *)obj2;
+        intptr_t i, len;
+#   include "mzeqchk.inc"
+        if ((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
+          return 0;
+        if (sv1->mask != sv2->mask)
+          return 0;
+        if (union_check(obj1, obj2, eql))
+          return 1;
+        len = scheme_stencil_vector_popcount(sv1->mask);
+        for (i = 0; i < len; i++) {
+          if (!is_equal(sv1->els[i], sv2->els[i], eql))
+            return 0;
+        }
+        return 1;
+      }
     case scheme_byte_string_type:
     case scheme_unix_path_type:
     case scheme_windows_path_type:
       {
         intptr_t l1, l2;
-        if ((eql->for_chaperone == 1) && (!SCHEME_IMMUTABLEP(obj1)
-                                          || !SCHEME_IMMUTABLEP(obj2)))
+        if (((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
+            && (!SCHEME_IMMUTABLEP(obj1) || !SCHEME_IMMUTABLEP(obj2)))
           return 0;
         l1 = SCHEME_BYTE_STRTAG_VAL(obj1);
         l2 = SCHEME_BYTE_STRTAG_VAL(obj2);
@@ -679,8 +762,8 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     case scheme_char_string_type:
       {
         intptr_t l1, l2;
-        if ((eql->for_chaperone == 1) && (!SCHEME_IMMUTABLEP(obj1)
-                                          || !SCHEME_IMMUTABLEP(obj2)))
+        if (((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
+            && (!SCHEME_IMMUTABLEP(obj1) || !SCHEME_IMMUTABLEP(obj2)))
           return 0;
         l1 = SCHEME_CHAR_STRTAG_VAL(obj1);
         l2 = SCHEME_CHAR_STRTAG_VAL(obj2);
@@ -706,18 +789,18 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
         st1 = SCHEME_STRUCT_TYPE(obj1);
         st2 = SCHEME_STRUCT_TYPE(obj2);
 
-        if (eql->for_chaperone == 1)
+        if (eql->mode == EQUAL_MODE_CHAPERONE_OF)
           procs1 = NULL;
         else
           procs1 = scheme_struct_type_property_ref(scheme_impersonator_of_property, (Scheme_Object *)st1);
         if (procs1)
-          procs1 = scheme_apply_impersonator_of(eql->for_chaperone, procs1, obj1);
-        if (eql->for_chaperone)
+          procs1 = scheme_apply_impersonator_of(eql->mode, procs1, obj1);
+        if ((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_IMPERSONATOR_OF))
           procs2 = NULL;
         else {
           procs2 = scheme_struct_type_property_ref(scheme_impersonator_of_property, (Scheme_Object *)st2);
           if (procs2)
-            procs2 = scheme_apply_impersonator_of(eql->for_chaperone, procs2, obj2);
+            procs2 = scheme_apply_impersonator_of(eql->mode, procs2, obj2);
         }
 
         if (procs1 || procs2) {
@@ -728,9 +811,11 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
         } else {
           /* don't discard `prop:impersonator-of` if checking for `impersonator-of?`
              or `chaperone-of?` */
-          if (eql->for_chaperone) {
+          if ((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_IMPERSONATOR_OF)) {
             procs2 = scheme_struct_type_property_ref(scheme_impersonator_of_property, (Scheme_Object *)st2);
-            if (procs2 && scheme_apply_impersonator_of(eql->for_chaperone, procs2, obj2))
+            if (procs2 && scheme_apply_impersonator_of(eql->mode, procs2, obj2))
+              /* Second argument is an impersonator, so
+                 `impersonator-of?` or `chaperone-of?` fails */
               return 0;
           }
           
@@ -744,10 +829,17 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
 
           if (procs1) {
             /* Has an equality property: */
-            Scheme_Object *a[3], *recur;
+            Scheme_Object *a[4], *recur;
             Equal_Info *eql2;
 #     include "mzeqchk.inc"
 
+            if (((eql->mode == EQUAL_MODE_EQUAL_ALWAYS) || (eql->mode == EQUAL_MODE_CHAPERONE_OF))
+                && !(MZ_OPT_HASH_KEY(&st1->iso) & STRUCT_TYPE_ALL_IMMUTABLE)
+                && (SCHEME_VEC_SIZE(procs1) != SCHEME_NEW_EQUAL_PROTOCOL_VECTOR_LENGTH)) {
+              /* Mutable records cannot use the old protocol for `chapeone-of?` or `equal-always?` */
+              return 0;
+            }
+
             if (union_check(obj1, obj2, eql))
               return 1;
 
@@ -770,17 +862,27 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
             a[1] = orig_obj2;
             a[2] = recur;
 
-            procs1 = SCHEME_VEC_ELS(procs1)[1];
-
-            recur = _scheme_apply(procs1, 3, a);
+            if (SCHEME_VEC_SIZE(procs1) == SCHEME_NEW_EQUAL_PROTOCOL_VECTOR_LENGTH) {
+              /* new protocol */
+              procs1 = SCHEME_VEC_ELS(procs1)[1];
+              a[3] = (((eql->mode == EQUAL_MODE_EQUAL_ALWAYS) || (eql->mode == EQUAL_MODE_CHAPERONE_OF))
+                      ? scheme_false
+                      : scheme_true);
+              recur = _scheme_apply(procs1, 4, a);
+            } else {
+              /* old protocol */
+              procs1 = SCHEME_VEC_ELS(procs1)[1];
+              recur = _scheme_apply(procs1, 3, a);              
+            }
 
             memcpy(eql, eql2, sizeof(Equal_Info));
 
             return SCHEME_TRUEP(recur);
           } else if (st1 != st2) {
             return 0;
-          } else if ((eql->for_chaperone == 1)
+          } else if (((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
                      && !(MZ_OPT_HASH_KEY(&st1->iso) & STRUCT_TYPE_ALL_IMMUTABLE)) {
+            /* Mutable records must be `eq?` for `chaperone-of?` and `equal-always?` */
             return 0;
           } else {
             /* Same types, but doesn't have an equality property
@@ -804,8 +906,8 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     case scheme_box_type:
       {
         SCHEME_USE_FUEL(1);
-        if ((eql->for_chaperone == 1) && (!SCHEME_IMMUTABLEP(obj1)
-                                          || !SCHEME_IMMUTABLEP(obj2)))
+        if (((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
+            && (!SCHEME_IMMUTABLEP(obj1) || !SCHEME_IMMUTABLEP(obj2)))
           return 0;
         if (union_check(obj1, obj2, eql))
           return 1;
@@ -819,10 +921,11 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
           obj2 = scheme_unbox(orig_obj2);
         goto top;
       }
+    /* normal mutable hash table */
     case scheme_hash_table_type:
       {
 #   include "mzeqchk.inc"
-        if (eql->for_chaperone == 1) 
+        if ((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
           return 0;
         if (union_check(obj1, obj2, eql))
           return 1;
@@ -830,9 +933,11 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
                                            (Scheme_Hash_Table *)obj2, orig_obj2,
                                            eql);
       }
+    /* functional hash tables */
     case scheme_hash_tree_type:
     case scheme_eq_hash_tree_type:
     case scheme_eqv_hash_tree_type:
+    case scheme_equal_always_hash_tree_type:
     case scheme_hash_tree_indirection_type:
       {
 #   include "mzeqchk.inc"
@@ -841,11 +946,12 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
         return scheme_hash_tree_equal_rec((Scheme_Hash_Tree *)obj1, orig_obj1,
                                           (Scheme_Hash_Tree *)obj2, orig_obj2,
                                           eql);
-      } 
+      }
+    /* old-style hash table, with buckets, mutable */
     case scheme_bucket_table_type:
       {
 #   include "mzeqchk.inc"
-        if (eql->for_chaperone == 1) 
+        if ((eql->mode == EQUAL_MODE_CHAPERONE_OF) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
           return 0;
         if (union_check(obj1, obj2, eql))
           return 1;
@@ -854,8 +960,8 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
                                              eql);
       }
     default:
-      if (!eql->for_chaperone && ((t1 == scheme_chaperone_type)
-                                  || (t1 == scheme_proc_chaperone_type))) {
+      if (((eql->mode == EQUAL_MODE_EQUAL) || (eql->mode == EQUAL_MODE_EQUAL_ALWAYS))
+          && ((t1 == scheme_chaperone_type) || (t1 == scheme_proc_chaperone_type))) {
         /* both chaperones */
         obj1 = ((Scheme_Chaperone *)obj1)->val;
         obj2 = ((Scheme_Chaperone *)obj2)->val;
@@ -986,7 +1092,7 @@ int scheme_chaperone_of(Scheme_Object *obj1, Scheme_Object *obj2)
   Equal_Info eql;
 
   init_equal_info(&eql);
-  eql.for_chaperone = 1;
+  eql.mode = 1; /* mode 1: 'chaperone-of? */
 
   return is_equal(obj1, obj2, &eql);
 }
@@ -996,12 +1102,12 @@ int scheme_impersonator_of(Scheme_Object *obj1, Scheme_Object *obj2)
   Equal_Info eql;
 
   init_equal_info(&eql);
-  eql.for_chaperone = 3;
+  eql.mode = 3; /* mode 3: 'impersonator-of? */
 
   return is_equal(obj1, obj2, &eql);
 }
 
-Scheme_Object *scheme_apply_impersonator_of(int for_chaperone, Scheme_Object *procs, Scheme_Object *obj)
+Scheme_Object *scheme_apply_impersonator_of(int mode, Scheme_Object *procs, Scheme_Object *obj)
 {
   Scheme_Object *a[1], *v, *oprocs;
 
@@ -1013,7 +1119,13 @@ Scheme_Object *scheme_apply_impersonator_of(int for_chaperone, Scheme_Object *pr
   
   oprocs = scheme_struct_type_property_ref(scheme_impersonator_of_property, v);  
   if (!oprocs || !SAME_OBJ(SCHEME_CAR(oprocs), SCHEME_CAR(procs)))
-    scheme_contract_error((for_chaperone ? "impersonator-of?" : "equal?"),
+    scheme_contract_error(((mode == EQUAL_MODE_CHAPERONE_OF)
+                           ? "chaperone-of?"
+                           : ((mode == EQUAL_MODE_IMPERSONATOR_OF)
+                              ? "impersonator-of?"
+                              : ((mode == EQUAL_MODE_EQUAL_ALWAYS)
+                                 ? "equal-always?"
+                                 : "equal?"))),
                           "impersonator-of property procedure returned a value with a different prop:impersonator-of source",
                           "original value", 1, obj,
                           "returned value", 1, v,
@@ -1024,7 +1136,13 @@ Scheme_Object *scheme_apply_impersonator_of(int for_chaperone, Scheme_Object *pr
   if (procs || oprocs)
     if (!procs || !oprocs || !SAME_OBJ(SCHEME_VEC_ELS(oprocs)[0], 
                                        SCHEME_VEC_ELS(procs)[0]))
-      scheme_contract_error((for_chaperone ? "impersonator-of?" : "equal?"),
+      scheme_contract_error(((mode == EQUAL_MODE_CHAPERONE_OF)
+                             ? "chaperone-of?"
+                             : ((mode == EQUAL_MODE_IMPERSONATOR_OF)
+                                ? "impersonator-of?"
+                                : ((mode == EQUAL_MODE_EQUAL_ALWAYS)
+                                   ? "equal-always?"
+                                   : "equal?"))),
                             "impersonator-of property procedure returned a value with a different prop:equal+hash source",
                             "original value", 1, obj,
                             "returned value", 1, v,
