@@ -140,6 +140,9 @@ Scheme_On_Atomic_Timeout_Proc (*scheme_set_on_atomic_timeout)(Scheme_On_Atomic_T
 /*========================================================================*/
 void (*scheme_signal_error)(const char *msg, ...);
 void (*scheme_raise_exn)(int exnid, ...);
+void (*scheme_raise_realm_exn)(int exnid,
+                                                  intptr_t name_len, Scheme_Object *name_realm, Scheme_Object *msg_realm,
+                                                  ...);
 void (*scheme_warning)(char *msg, ...);
 void (*scheme_raise)(Scheme_Object *exn);
 int (*scheme_log_level_p)(Scheme_Logger *logger, int level);
@@ -165,6 +168,7 @@ void (*scheme_wrong_count_m)(const char *name, int minc, int maxc, int argc, Sch
 void (*scheme_case_lambda_wrong_count)(const char *name, int argc, Scheme_Object **argv, int is_method, int count, ...);
 void (*scheme_wrong_type)(const char *name, const char *expected, int which, int argc, Scheme_Object **argv);
 void (*scheme_wrong_contract)(const char *name, const char *expected, int which, int argc, Scheme_Object **argv);
+void (*scheme_wrong_contract_for_realm)(const char *name, Scheme_Object *realm, const char *expected, int which, int argc, Scheme_Object **argv);
 void (*scheme_wrong_field_type)(Scheme_Object *c_name, const char *expected, Scheme_Object *o);
 void (*scheme_wrong_field_contract)(Scheme_Object *c_name, const char *expected, Scheme_Object *o);
 void (*scheme_arg_mismatch)(const char *name, const char *msg, Scheme_Object *o);
@@ -200,6 +204,7 @@ Scheme_Object *scheme_tail_call_waiting;
 Scheme_Object *scheme_multiple_values;
 unsigned short **scheme_uchar_table;
 unsigned char **scheme_uchar_cases_table;
+unsigned char **scheme_uchar_graphbreaks_table;
 unsigned char **scheme_uchar_cats_table;
 int *scheme_uchar_ups;
 int *scheme_uchar_downs;
@@ -270,6 +275,9 @@ Scheme_Object *(*scheme_extract_one_cc_mark)(Scheme_Object *mark_set,
 Scheme_Object *(*scheme_extract_one_cc_mark_to_tag)(Scheme_Object *mark_set, 
                                                            Scheme_Object *key,
                                                            Scheme_Object *prompt_tag);
+Scheme_Object *(*scheme_extract_cc_mark_list)(Scheme_Object *mark_set, 
+                                                     Scheme_Object *key,
+                                                     Scheme_Object *prompt_tag);
 /* Internal */
 Scheme_Object *(*scheme_do_eval)(Scheme_Object *obj, int _num_rands, Scheme_Object **rands, int val);
 void (*scheme_detach_multple_array)(Scheme_Object **a);
@@ -508,6 +516,7 @@ Scheme_Object *(*scheme_make_external_cptr)(void *cptr, Scheme_Object *typetag);
 Scheme_Object *(*scheme_make_offset_external_cptr)(void *cptr, intptr_t offset, Scheme_Object *typetag);
 int (*scheme_is_cpointer)(Scheme_Object *cp);
 const char *(*scheme_get_proc_name)(Scheme_Object *p, int *len, int for_error);
+Scheme_Object *p;
 /*========================================================================*/
 /*                               strings                                  */
 /*========================================================================*/
