@@ -21,6 +21,10 @@
 #include "boot.h"
 #include "api.h"
 
+#ifdef PBCHUNK_REGISTER
+static void register_pbchunks();
+#endif
+
 #define RACKET_AS_BOOT
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
@@ -122,6 +126,10 @@ void racket_boot(racket_boot_arguments_t *ba)
 
   Sscheme_init(NULL);
 
+#ifdef PBCHUNK_REGISTER
+  register_pbchunks();
+#endif
+  
   if ((ba->argc == 4) && !strcmp(ba->argv[0], "--cross-server"))
     cross_server = 1;
 
@@ -233,6 +241,74 @@ void racket_boot(racket_boot_arguments_t *ba)
 
 /* **************************************** */
 
+#ifdef PBCHUNK_REGISTER
+extern void register_petite0_pbchunks();
+extern void register_petite1_pbchunks();
+extern void register_petite2_pbchunks();
+extern void register_petite3_pbchunks();
+extern void register_petite4_pbchunks();
+extern void register_petite5_pbchunks();
+extern void register_petite6_pbchunks();
+extern void register_petite7_pbchunks();
+extern void register_petite8_pbchunks();
+extern void register_petite9_pbchunks();
+extern void register_scheme0_pbchunks();
+extern void register_scheme1_pbchunks();
+extern void register_scheme2_pbchunks();
+extern void register_scheme3_pbchunks();
+extern void register_scheme4_pbchunks();
+extern void register_scheme5_pbchunks();
+extern void register_scheme6_pbchunks();
+extern void register_scheme7_pbchunks();
+extern void register_scheme8_pbchunks();
+extern void register_scheme9_pbchunks();
+extern void register_racket0_pbchunks();
+extern void register_racket1_pbchunks();
+extern void register_racket2_pbchunks();
+extern void register_racket3_pbchunks();
+extern void register_racket4_pbchunks();
+extern void register_racket5_pbchunks();
+extern void register_racket6_pbchunks();
+extern void register_racket7_pbchunks();
+extern void register_racket8_pbchunks();
+extern void register_racket9_pbchunks();
+
+static void register_pbchunks() {
+  register_petite0_pbchunks();
+  register_petite1_pbchunks();
+  register_petite2_pbchunks();
+  register_petite3_pbchunks();
+  register_petite4_pbchunks();
+  register_petite5_pbchunks();
+  register_petite6_pbchunks();
+  register_petite7_pbchunks();
+  register_petite8_pbchunks();
+  register_petite9_pbchunks();
+  register_scheme0_pbchunks();
+  register_scheme1_pbchunks();
+  register_scheme2_pbchunks();
+  register_scheme3_pbchunks();
+  register_scheme4_pbchunks();
+  register_scheme5_pbchunks();
+  register_scheme6_pbchunks();
+  register_scheme7_pbchunks();
+  register_scheme8_pbchunks();
+  register_scheme9_pbchunks();
+  register_racket0_pbchunks();
+  register_racket1_pbchunks();
+  register_racket2_pbchunks();
+  register_racket3_pbchunks();
+  register_racket4_pbchunks();
+  register_racket5_pbchunks();
+  register_racket6_pbchunks();
+  register_racket7_pbchunks();
+  register_racket8_pbchunks();
+  register_racket9_pbchunks();
+}
+#endif
+
+/* **************************************** */
+
 enum {
   EMBEDDED_ENTRY_APPLY,
   EMBEDDED_ENTRY_PRIMITIVE_LOOKUP,
