@@ -18,6 +18,11 @@
 #define CENTRALIZED_SIGCHILD
 #endif
 
+#ifdef RKTIO_SYSTEM_UNIX
+static void reliably_copy_or_move_fd(int src_fd, int target_fd, int move);
+static void close_non_standard_fd(int fd);
+#endif
+
 /*========================================================================*/
 /* Process data structure                                                 */
 /*========================================================================*/
@@ -352,7 +357,7 @@ void centralized_done_with_process_id(int pid, int in_group)
         if (keep_unused) {
           st->next_unused = unused_pid_statuses;
           unused_pid_statuses = st;
-          if (st->signal_fd)
+          if (st->signal_fd && st->in_group)
             remove_group_signal_fd(st->signal_fd);
         } else
           st->unneeded = 1;
@@ -493,8 +498,10 @@ static void add_group_signal_fd(rktio_signal_handle_t *signal_fd)
     signal_fd_count = (signal_fd_count + 4) * 2;
     a = malloc(sizeof(Group_Signal_FD) * signal_fd_count);
     memset(a, 0, sizeof(Group_Signal_FD) * signal_fd_count);
-    memcpy(a, signal_fds, sizeof(Group_Signal_FD) * count);
-    if (signal_fds) free(signal_fds);
+    if (signal_fds) {
+      memcpy(a, signal_fds, sizeof(Group_Signal_FD) * count);
+      free(signal_fds);
+    }
     signal_fds = a;
   }
 
@@ -595,7 +602,6 @@ static void init_sigchld(rktio_t *rktio)
 #endif
 }
 
-#if !defined(CENTRALIZED_SIGCHILD)
 static void remove_from_sigchld_chain(rktio_t *rktio)
 {
   if (rktio->in_sigchld_chain) {
@@ -613,7 +619,6 @@ static void remove_from_sigchld_chain(rktio_t *rktio)
     }
   }
 }
-#endif
 
 static void check_child_done(rktio_t *rktio, pid_t pid)
 {
@@ -941,14 +946,25 @@ void rktio_process_forget(rktio_t *rktio, rktio_process_t *sp)
     centralized_ended_child();
   }
 # else
-  if (!((System_Child *)sp->handle)->done) {
+  System_Child *sc = (System_Child *)sp->handle, *sc2, *prev;
+  if (!sc->done) {
     void **unused_pid;
     unused_pid = malloc(sizeof(void *) * 2);
     unused_pid[0] = (void *)(intptr_t)sp->pid;
     unused_pid[1] = unused_pids;
     rktio->need_to_check_children = 1;
+
+    prev = NULL;
+    for (sc2 = rktio->system_children; sc2; prev = sc2, sc2 = sc2->next) {
+      if (sc2 == sc) {
+        if (prev)
+          prev->next = sc->next;
+        else
+          rktio->system_children = sc->next;
+      }
+    }
   }
-  free(sp->handle);
+  free(sc);
 # endif
 #endif
 
@@ -966,6 +982,10 @@ int rktio_process_init(rktio_t *rktio)
      it's a per-thread setting on Linux, and we want SIGCHLD blocked everywhere. */
   block_sigchld();
 
+  /* We'll set a handler later (possibly after other threads start),
+     so register SIGCHLD now: */
+  rktio_will_modify_os_signal_handler(SIGCHLD);
+
   centralized_start_child_signal_handler();
 #endif
 
@@ -1048,18 +1068,58 @@ static char *cmdline_protect(const char *s)
   return MSC_IZE(strdup)(s);
 }
 
+/* Avoid direct reference to Vista functionality: */
+typedef struct {
+  STARTUPINFOW StartupInfo;
+  void *lpAttributeList;
+} rktio_STARTUPINFOEXW;
+#define rktio_EXTENDED_STARTUPINFO_PRESENT 0x00080000
+#define rktio_PROC_THREAD_ATTRIBUTE_HANDLE_LIST 0x00020002
+typedef BOOL (*rktio_InitializeProcThreadAttributeList_t)(void *lpAttributeList,
+                                                          DWORD dwAttributeCount,
+                                                          DWORD dwFlags,
+                                                          PSIZE_T lpSize);
+static rktio_InitializeProcThreadAttributeList_t rktio_InitializeProcThreadAttributeList;
+typedef BOOL (*rktio_UpdateProcThreadAttribute_t)(void *lpAttributeList,
+                                                  DWORD dwFlags,
+                                                  DWORD_PTR Attribute,
+                                                  PVOID lpValue,
+                                                  SIZE_T cbSize,
+                                                  PVOID lpPreviousValue,
+                                                  PSIZE_T lpReturnSize);
+static rktio_UpdateProcThreadAttribute_t rktio_UpdateProcThreadAttribute;
+typedef void (*rktio_DeleteProcThreadAttributeList_t)(void *lpAttributeList);
+static rktio_DeleteProcThreadAttributeList_t rktio_DeleteProcThreadAttributeList;
+static void init_thread_attr_procs()
+{
+  if (!rktio_InitializeProcThreadAttributeList
+      || !rktio_UpdateProcThreadAttribute) {
+    HMODULE hm;
+
+    hm = LoadLibraryW(L"kernel32.dll");
+
+    rktio_InitializeProcThreadAttributeList = (rktio_InitializeProcThreadAttributeList_t)GetProcAddress(hm, "InitializeProcThreadAttributeList");
+    rktio_UpdateProcThreadAttribute = (rktio_UpdateProcThreadAttribute_t)GetProcAddress(hm, "UpdateProcThreadAttribute");
+    rktio_DeleteProcThreadAttributeList = (rktio_DeleteProcThreadAttributeList_t)GetProcAddress(hm, "DeleteProcThreadAttributeList");
+
+    FreeLibrary(hm);
+  }
+}
+
 static intptr_t do_spawnv(rktio_t *rktio,
                           const char *command, int argc, const char * const *argv,
 			  int exact_cmdline, intptr_t sin, intptr_t sout, intptr_t serr, int *pid,
 			  int new_process_group, int chain_termination_here_to_child,
-                          void *env, const char *wd)
+                          void *env, const char *wd,
+                          int disable_inherit)
 {
-  intptr_t i, l, len = 0;
+  intptr_t i, l, len = 0, retval;
   int use_jo;
   intptr_t cr_flag;
   char *cmdline;
   wchar_t *cmdline_w, *wd_w, *command_w;
-  STARTUPINFOW startup;
+  rktio_STARTUPINFOEXW startupx;
+  STARTUPINFOW *startup;
   PROCESS_INFORMATION info;
 
   if (exact_cmdline) {
@@ -1082,18 +1142,19 @@ static intptr_t do_spawnv(rktio_t *rktio,
     cmdline[len] = 0;
   }
 
-  memset(&startup, 0, sizeof(startup));
-  startup.cb = sizeof(startup);
-  startup.dwFlags = STARTF_USESTDHANDLES;
-  startup.hStdInput = (HANDLE)sin;
-  startup.hStdOutput = (HANDLE)sout;
-  startup.hStdError = (HANDLE)serr;
+  memset(&startupx, 0, sizeof(startupx));
+  startup = &startupx.StartupInfo;
+  startup->cb = sizeof(*startup);
+  startup->dwFlags = STARTF_USESTDHANDLES;
+  startup->hStdInput = (HANDLE)sin;
+  startup->hStdOutput = (HANDLE)sout;
+  startup->hStdError = (HANDLE)serr;
 
   /* If none of the stdio handles are consoles, specifically
      create the subprocess without a console: */
-  if (!rktio_system_fd_is_terminal(rktio, (intptr_t)startup.hStdInput)
-      && !rktio_system_fd_is_terminal(rktio, (intptr_t)startup.hStdOutput)
-      && !rktio_system_fd_is_terminal(rktio, (intptr_t)startup.hStdError))
+  if (!rktio_system_fd_is_terminal(rktio, (intptr_t)startup->hStdInput)
+      && !rktio_system_fd_is_terminal(rktio, (intptr_t)startup->hStdOutput)
+      && !rktio_system_fd_is_terminal(rktio, (intptr_t)startup->hStdError))
     cr_flag = CREATE_NO_WINDOW;
   else
     cr_flag = 0;
@@ -1126,25 +1187,61 @@ static intptr_t do_spawnv(rktio_t *rktio,
   wd_w = WIDE_PATH_copy(wd);
   command_w = WIDE_PATH_temp(command);
 
+  if (disable_inherit) {
+    /* don't just set the `bInherit` argument to `CreateProcessW` to
+       false, because that disables sharing for
+       stdin.stdout/stderr. */
+    init_thread_attr_procs();
+    if (rktio_InitializeProcThreadAttributeList
+        && rktio_UpdateProcThreadAttribute
+        && rktio_DeleteProcThreadAttributeList) {
+      LPPROC_THREAD_ATTRIBUTE_LIST lpAttributeList = NULL;
+      SIZE_T size = 0;
+      HANDLE handles_to_inherit[3];
+      rktio_InitializeProcThreadAttributeList(NULL, 1, 0, &size);
+      lpAttributeList = HeapAlloc(GetProcessHeap(), 0, size);
+      rktio_InitializeProcThreadAttributeList(lpAttributeList, 1, 0, &size);
+      handles_to_inherit[0] = startup->hStdInput;
+      handles_to_inherit[1] = startup->hStdOutput;
+      handles_to_inherit[2] = startup->hStdError;
+      rktio_UpdateProcThreadAttribute(lpAttributeList,
+                                      0, rktio_PROC_THREAD_ATTRIBUTE_HANDLE_LIST,
+                                      handles_to_inherit,
+                                      sizeof(handles_to_inherit), NULL, NULL);
+      startupx.lpAttributeList = lpAttributeList;
+      startup->cb = sizeof(startupx);
+      cr_flag |= rktio_EXTENDED_STARTUPINFO_PRESENT;
+    } else
+      disable_inherit = 0;
+  }
+
   if (cmdline_w
       && wd_w
       && command_w
       && CreateProcessW(command_w, cmdline_w, 
                         NULL, NULL, 1 /*inherit*/,
                         cr_flag, env, wd_w,
-                        &startup, &info)) {
+                        startup, &info)) {
     if (use_jo)
       AssignProcessToJobObject(rktio->process_job_object, info.hProcess);
     CloseHandle(info.hThread);
     *pid = info.dwProcessId;
     free(cmdline_w);
     free(wd_w);
-    return (intptr_t)info.hProcess;
+    retval = (intptr_t)info.hProcess;
   } else {
     if (cmdline_w) free(cmdline_w);
     if (wd_w) free(wd_w);
-    return -1;
+    retval = -1;
   }
+
+  if (disable_inherit) {
+    void *lpAttributeList = startupx.lpAttributeList;
+    rktio_DeleteProcThreadAttributeList(lpAttributeList);
+    HeapFree(GetProcessHeap(), 0, lpAttributeList);
+  }
+
+ return retval;
 }
 
 static void CopyFileHandleForSubprocess(intptr_t *hs, int pos)
@@ -1224,6 +1321,9 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
   rktio_process_t *subproc;
 #if defined(RKTIO_SYSTEM_WINDOWS)
   intptr_t spawn_status;
+#else
+  rktio_const_string_t *new_argv;
+  int close_after_len;
 #endif
   int new_process_group = (flags & RKTIO_PROCESS_NEW_GROUP);
   int stderr_is_stdout = (flags & RKTIO_PROCESS_STDOUT_AS_STDERR);
@@ -1274,8 +1374,17 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
 
   if (envvars)
     env = rktio_envvars_to_block(rktio, envvars);
-  else
+  else {
+#ifdef RKTIO_USE_PTHREADS
+    {
+      rktio_envvars_t *current_envvars = rktio_envvars(rktio);
+      env = rktio_envvars_to_block(rktio, current_envvars);
+      rktio_envvars_free(rktio, current_envvars);
+    }
+#else
     env = NULL;
+#endif
+  }
 
 #if defined(RKTIO_SYSTEM_WINDOWS)
 
@@ -1311,7 +1420,8 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
 			     &pid,
                              new_process_group,
                              windows_chain_termination_to_child,
-                             env, current_directory);
+                             env, current_directory,
+                             flags & RKTIO_PROCESS_NO_INHERIT_FDS);
 
     if (!windows_exact_cmdline) {
       for (i = 0; i < argc; i++) {
@@ -1345,6 +1455,21 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
     init_sigchld(rktio);
 #endif
 
+    if (flags & RKTIO_PROCESS_NO_CLOSE_FDS)
+      close_after_len = 0;
+    else
+      close_after_len = rktio_close_fds_len();
+
+    /* add a NULL terminator */
+    {
+      int i;
+      new_argv = malloc(sizeof(char *) * (argc + 1));
+      for (i = 0; i < argc; i++) {
+        new_argv[i] = argv[i];
+      }
+      new_argv[i] = NULL;
+    }
+
 #if defined(__QNX__)
     pid = vfork();
 #elif defined(SUBPROCESS_USE_FORK1)
@@ -1353,8 +1478,6 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
     pid = fork();
 #endif
 
-    
-
     if (pid > 0) {
       /* This is the original process, which needs to manage the 
          newly created child process. */
@@ -1437,44 +1560,69 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
 		  
       if (env)
         free(env);
+      free(new_argv);
+#if !defined(CENTRALIZED_SIGCHILD)
+      free(sc);
+#endif
 
       return NULL;
 
     case 0: /* child */
 
       {
-        int errid;
-        
-	/* Copy pipe descriptors to stdin and stdout */
-	do {
-	  errid = MSC_IZE(dup2)(to_subprocess[0], 0);
-	} while (errid == -1 && errno == EINTR);
-	do {
-	  errid = MSC_IZE(dup2)(from_subprocess[1], 1);
-	} while (errid == -1 && errno == EINTR);
-	do {
-	  errid = MSC_IZE(dup2)(err_subprocess[1], 2);
-	} while (errid == -1 && errno == EINTR);
+	int in_fd, out_fd, err_fd;
+	in_fd = to_subprocess[0];
+	out_fd = from_subprocess[1];
+	err_fd = err_subprocess[1];
+
+	/* Make sure out/err descriptors don't get clobbered by moving
+	   them if they're occupying a file descriptor we need to move
+	   a different descriptor into. (Note that while it's unlikely
+	   that input/output file descriptors will be the same, it
+	   isn't impossible, so handle that, too.) */
+	if ((err_fd == 0 && err_fd != in_fd)
+	    || (err_fd == 1 && err_fd != out_fd)) {
+	  int new_err_fd = 2;
+	  while (new_err_fd == in_fd || new_err_fd == out_fd) {
+	    new_err_fd++;
+	  }
+	  reliably_copy_or_move_fd(err_fd, new_err_fd, err_fd != in_fd);
+	  if (out_fd == err_fd) {
+	    out_fd = new_err_fd;
+	  }
+	  err_fd = new_err_fd;
+	}
+	if (out_fd == 0 && out_fd != in_fd) {
+	  int new_out_fd = 1;
+	  while (new_out_fd == in_fd || new_out_fd == err_fd) {
+	    new_out_fd++;
+	  }
+	  reliably_copy_or_move_fd(out_fd, new_out_fd, out_fd != in_fd);
+	  out_fd = new_out_fd;
+	}
+
+	/* Assign new process stdin/stdout/stderr, closing old
+	   descriptors when not used by some other descriptor */
+	reliably_copy_or_move_fd(in_fd, 0, in_fd != out_fd && in_fd != err_fd);
+	reliably_copy_or_move_fd(out_fd, 1, out_fd > 0 && out_fd != err_fd);
+	reliably_copy_or_move_fd(err_fd, 2, err_fd > 1);
 
 	/* Close unwanted descriptors */
 	if (!stdin_fd) {
-	  rktio_reliably_close(to_subprocess[0]);
-	  rktio_reliably_close(to_subprocess[1]);
+	  close_non_standard_fd(to_subprocess[1]);
 	}
 	if (!stdout_fd) {
-	  rktio_reliably_close(from_subprocess[0]);
-	  rktio_reliably_close(from_subprocess[1]);
+	  close_non_standard_fd(from_subprocess[0]);
 	}
-	if (!stderr_fd) {
-          if (!stderr_is_stdout) {
-            rktio_reliably_close(err_subprocess[0]);
-            rktio_reliably_close(err_subprocess[1]);
-          }
+	if (!stderr_fd && !stderr_is_stdout) {
+	  close_non_standard_fd(err_subprocess[0]);
 	}
 
-        rktio_close_fds_after_fork(0, 1, 2);
+        rktio_close_fds_after_fork(close_after_len, 0, 1, 2);
       }
 
+      rktio_restore_modified_signal_handlers();
+
       /* Set real CWD: */
       if (!rktio_set_current_directory(rktio, current_directory)) {
         fprintf(stderr, "racket: chdir failed to: %s\n", current_directory);
@@ -1484,25 +1632,21 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
       /* Exec new process */
 
       {
-	int err, i;
-        rktio_const_string_t *new_argv;
-
-        /* add a NULL terminator */
-        new_argv = malloc(sizeof(char *) * (argc + 1));
-        for (i = 0; i < argc; i++) {
-          new_argv[i] = argv[i];
-        }
-        new_argv[i] = NULL;
+	int err;
+        char **use_env;
 
         if (!env)
-          env = rktio_get_environ_array();
-        
-	err = MSC_IZE(execve)(command, (char **)new_argv, (char **)env);
+          use_env = rktio_get_environ_array();
+        else
+          use_env = env;
+
+	err = MSC_IZE(execve)(command, (char **)new_argv, use_env);
         if (err)
           err = errno;
 
         if (env)
           free(env);
+        free(new_argv);
 
 	/* If we get here it failed; give up */
 
@@ -1515,6 +1659,8 @@ rktio_process_result_t *rktio_process(rktio_t *rktio,
 
     default: /* parent */
 
+      free(new_argv);
+
       break;
     }
 #endif
@@ -1585,10 +1731,32 @@ int rktio_process_pid(rktio_t *rktio, rktio_process_t *sp)
 }
 
 #ifdef RKTIO_SYSTEM_UNIX
-void rktio_close_fds_after_fork(int skip1, int skip2, int skip3)
+static void reliably_copy_or_move_fd(int src_fd, int target_fd, int move)
+{
+  if (src_fd != target_fd) {
+    int errid;
+    do {
+      errid = MSC_IZE(dup2)(src_fd, target_fd);
+    } while (errid == -1 && errno == EINTR);
+    if (move) {
+      rktio_reliably_close(src_fd);
+    }
+  }
+}
+
+static void close_non_standard_fd(int fd)
+{
+  if (fd != 0 && fd != 1 && fd != 2) {
+    rktio_reliably_close(fd);
+  }
+}
+
+int rktio_close_fds_len()
 {
   int i;
 
+  /* These functions are not async-signal safe, so use them before
+     a fork: */
 # ifdef USE_ULIMIT
   i = ulimit(4, 0);
 # elif defined(__ANDROID__)
@@ -1596,12 +1764,18 @@ void rktio_close_fds_after_fork(int skip1, int skip2, int skip3)
 # else
   i = getdtablesize();
 # endif
+
+  return i;
+}
+
+void rktio_close_fds_after_fork(int i, int skip1, int skip2, int skip3)
+{
+  /* incoming `i` should be the result of `rktio_close_fds_len` before
+     a fork */
+
   while (i--) {
-    int cr;
     if ((i != skip1) && (i != skip2) && (i != skip3)) {
-      do {
-        cr = close(i);
-      } while ((cr == -1) && (errno == EINTR));
+      rktio_reliably_close(i);
     }
   }
 }
